/* boss.c

Port of boss from VMS Pascal into C, for Linux. Steve Kertes, 1999-2000.
Here is the original copyright notice from boss.pas:

{ Moria Version 5.0     COPYRIGHT (c) Robert Alan Koeneke
                        Public Domain
                   Modified EXTENSIVELY by:
                    Matthew W. Koch   -MWK
                   with minor help from:
                    Kendall R. Sears  -Opusii
                    Russell E. Billings -REB
                    David G. Strubel    -DGS
 
        I lovingly dedicate this game to hackers and adventurers
        everywhere...
 
        Designer and Programmer : Robert Alan Koeneke
                                  University of Oklahoma
        Assitant Programmer     : Jimmey Wayne Todd
                                  University of Oklahoma
 
        Moria may be copied and modified freely as long as the above
        credits are retained.  No one who-so-ever may sell or market
        this software in any form without the expressed written consent
        of the author Robert Alan Koeneke.
 
_______________________________________________________________________
BOSS version 1.0  by Robert Gulledge
		  and jason black
}
*/

#define __BOSS_C__
#include "boss.h"

//////////////////////////////////////////////////////////////////////

int	main(int argc, char *argv[], char *envp[])
{
#ifdef __CYGWIN__
  setenv("TERMINFO", "terminfo", 1);
#endif

  /* { Initialize, restore, and get the ball rolling. } */

  /* { SYSPRV stays off except when needed...} */
  game_state = GS_LOADING;
  init_priv_switch();
  priv_switch(0);

  /* { Check the terminal type and see if it is supported } */
  termdef();

  /* { Get the directory location of the image } */
  get_paths();
 
  /* { Setup pause time for IO setup_io_pause; } */

  /* { Check to see if user is a wiz, scum, or just annoying } */
  user_name(username);
  look_at_userid();

  /* { Some neccesary initializations } */

  msg_line       = 1;
  quart_height   = trunc(SCREEN_HEIGHT/4);
  quart_width    = trunc(SCREEN_WIDTH /4);
  dun_level      = 0;
  turn           = 5760;    /* {8:00 a.m.} */

  if (putzuser) {
    wierd_chance   = 2160;  /*{make it a really wierd game...} */
  } else {
    wierd_chance   = 8640;
  }
  
  /* { Check or create hours.dat, print message } */
  intro(finam, argc, argv);
  clear_screen();

  /* { Init an IO channel for QIO } */
  /* init_channel(); */
  
  /* { Grab a random seed from the clock } */
  seed = get_seed();
  
  /* {Read in the monster and object data files.} */
  if (!read_data()) {
    exit_game();
  }
  
  /* { Sort the objects by level } */
  sort_objects();
  
  /* { Init monster and treasure levels for allocate } */
  init_m_level();
  init_t_level();
  
  /* { Init the store inventories } */
  store_init();

  if (COST_ADJ != 1.00) {
    price_adjust();
  }
  
  /* { Generate a character, or retrieve old one...  } */
  if (strlen(finam) > 0) {
    /* { Retrieve character } */
    game_state = GS_IGNORE_CTRL_C;
    generate = get_char(finam,true);
    change_name();
    magic_init(randes_seed);
  } else {
    /* { Create character } */
    create_character();
    char_inven_init();
    
    if (PM.pskill == SKILL_SCIENTIST ||
	PM.pskill == SKILL_SOLDIER   ||
	PM.pskill == SKILL_JOURNALIST) {
      learn_spell(&msg_flag);
      gain_mana(int_adj());
    }
    
    if (PM.pskill == SKILL_GURU ||
	PM.pskill == SKILL_WANDERER) {
      learn_prayer();
      gain_mana(wis_adj());
    }
    
    if (PM.pskill == SKILL_CONARTIST ||
	PM.pskill == SKILL_THIEF) {
      learn_extra(&msg_flag);
      gain_mana(chr_adj());
    }
    
    PM.cmana = PM.mana;
    randes_seed = seed;            /* { Description seed } */
    town_seed   = seed;            /* { Town generation seed  } */
    magic_init(randes_seed);
    generate = true;
  }
  
  /* { begin the game } */

  //with py.misc do;     { This determines the maximum player experience }
  player_max_exp = trunc(player_exp[MAX_PLAYER_LEVEL-1]*PM.expfact);

  clear_screen();
  prt_stat_block();
  
  /* { Loop till dead, or exit }*/
  do {
    
    if (generate) {
      generate_cave(); /*{ New level }*/
    }
    
    dungeon(); /*{ Dungeon logic-located in Main.Inc }*/
    generate = true;
    
  } while (!death);
  
  upon_death(); /*{ Character gets buried }*/
  
  return 0;
} /* end main */

//////////////////////////////////////////////////////////////////////

/* end boss.c */
