/* datafiles.c */
/**/

#include "boss.h"

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
static boolean read_creatures()
{
  /*{ Read in the creature data from an external file }*/

  integer       count;
  worlint       i1, i2, i3;
  vtype         in_line;
  FILE         *infile;
  boolean       return_value = true;

  //{read creatures from external file}
  infile = priv_fopen( BOSS_MON, "r");
  if (infile == NULL) {
    return_value = false;
  } else {

    in_line[0] = 0;
    fgets(in_line, sizeof(in_line), infile);
    sscanf(in_line,"%ld", &max_creatures);

    if (max_creatures > MAX_CREATURES) {
      return_value = false;
    } else {

      for (count = 1 ; count <= max_creatures ; count++) {
	//with c_list[count]. do;

	fgets(in_line, sizeof(in_line), infile);
	strtrim(in_line);
	strncpy(c_list[count].name, in_line, sizeof(c_list[count].name));
	c_list[count].name[sizeof(c_list[count].name)] = 0;
								   
	fgets(in_line, sizeof(in_line), infile);
	sscanf(in_line, "%x", &(c_list[count].cmove));
	fgets(in_line, sizeof(in_line), infile);
	sscanf(in_line, "%x", &(c_list[count].spells));
	fgets(in_line, sizeof(in_line), infile);
	sscanf(in_line, "%x", &(c_list[count].cdefense));
	
	fgets(in_line, sizeof(in_line), infile);
	sscanf(in_line, "%hd %hd %hd %hd %hd\n",
	       &(c_list[count].sleep),
	       &(c_list[count].mexp),
	       &i1,
	       &i2,
	       &i3  );
	c_list[count].aaf   = i1;
	c_list[count].ac    = i2;
	c_list[count].speed = i3;
	
	fgets(in_line, sizeof(in_line), infile);
	sscanf(in_line, "%ld %ld %ld %ld %ld\n",
	       &(c_list[count].reprod),
	       &(c_list[count].size),
	       &(c_list[count].tt),
	       &(c_list[count].intel),
	       &(c_list[count].speaks)   );
	
	fgets(in_line, sizeof(in_line), infile);
	c_list[count].cchar = in_line[0];
	
	fgets(in_line, sizeof(in_line), infile);
	strtrim(in_line);
	strcpy(c_list[count].hd, in_line);
	
	fgets(in_line, sizeof(in_line), infile);
	strtrim(in_line);
	strcpy(c_list[count].damage, in_line);
	
	fgets(in_line, sizeof(in_line), infile);
	sscanf(in_line, "%hd\n", &i1);
	c_list[count].level = i1;
      }
    }
    
    fclose(infile);
  }

  if (!return_value) {
    prt ("ERROR: A severe error occurred in read_creatures.",1,1);
    prt ("Please notify the BOSS Master...",2,1);
    pause_line(24);
  }

  return return_value;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
static boolean read_objects()
{
  /*{ Read in the master object list. It's about time someone did this. -RLG}*/

  integer    count;
  worlint    i1, i2;
  FILE      *infile;
  vtype      in_line;
  boolean    return_value = true;

  /*{ Read objects from external file }*/
  infile = priv_fopen( BOSS_OBJ, "r");
  if (infile == NULL) {
    return_value = false;
  } else {

    in_line[0] = 0;
    fgets(in_line, sizeof(in_line), infile);
    sscanf(in_line,"%ld", &max_objects);

    if (max_objects > MAX_OBJECTS) {
      return_value = false;
    } else {

      for (count = 1 ; count <= max_objects ; count++) {
	
	object_ident[count] = false; /*{set all objects to be not identified}*/

	//with object_list[count]. do;
	
	fgets(in_line, sizeof(in_line), infile);  /* comment line */

	fgets(in_line, sizeof(in_line), infile);
	in_line[strlen(in_line)-1] = 0;
	strcpy(object_list[count].name, in_line);

	fgets(in_line, sizeof(in_line), infile);
	sscanf(in_line, "%x", &(object_list[count].flags));
	
	fgets(in_line, sizeof(in_line), infile);
	sscanf(in_line, "%hd %ld %ld %hd %hd %hd\n",
	       &i1,
	       &(object_list[count].subval),
	       &(object_list[count].cost),
	       &(object_list[count].weight),
	       &(object_list[count].number),
	       &i2  );
	object_list[count].tval  = i1;
	object_list[count].level = i2;
	
	fgets(in_line, sizeof(in_line), infile);
	sscanf(in_line, "%ld %hd %hd %hd %hd\n",
	       &(object_list[count].p1),
	       &(object_list[count].tohit),
	       &(object_list[count].todam),
	       &(object_list[count].ac),
	       &(object_list[count].toac)  );
	
	fgets(in_line, sizeof(in_line), infile);
	in_line[strlen(in_line)-1] = 0;
	strcpy(object_list[count].damage, in_line);
	
	fgets(in_line, sizeof(in_line), infile);
	object_list[count].tchar = in_line[0];
	
      }
    }
      
    fclose(infile);
  }

  if (!return_value) {
    prt ("ERROR: A severe error occurred in read_objects.",1,1);
    prt ("Please notify the BOSS Master...",2,1);
    pause_line(24);
  }

  return return_value;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
static boolean read_inventory_list()
{
  /*{The obvious sequel. -RLG}*/

  integer    count;
  worlint    i1, i2;
  FILE      *infile;
  vtype      in_line;
  boolean    return_value = true;

  /*{ Read store inventory objects from external file }*/
  infile = priv_fopen( BOSS_INV, "r");
  if (infile == NULL) {
    return_value = false;
  } else {

    in_line[0] = 0;
    fgets(in_line, sizeof(in_line), infile);
    sscanf(in_line,"%ld", &inven_init_max);

    if (inven_init_max > MAX_INVENTORY_INIT) {
      return_value = false;
    } else {
      for (count = 1 ; count <= inven_init_max ; count++) {
	//with inventory_init[count]. do;

	fgets(in_line, sizeof(in_line), infile);  /* dummy line */
	fgets(in_line, sizeof(in_line), infile);
	in_line[strlen(in_line)-1] = 0;           /* strip newline */
	strcpy(inventory_init[count].name, in_line);

	fgets(in_line, sizeof(in_line), infile);
	sscanf(in_line, "%x", &(inventory_init[count].flags));
	
	fgets(in_line, sizeof(in_line), infile);
	sscanf(in_line, "%hd %ld %ld %hd %hd %hd\n",
	       &i1,
	       &(inventory_init[count].subval),
	       &(inventory_init[count].cost),
	       &(inventory_init[count].weight),
	       &(inventory_init[count].number),
	       &i2 );
	inventory_init[count].tval  = i1;
	inventory_init[count].level = i2;	

	fgets(in_line, sizeof(in_line), infile);
	sscanf(in_line, "%ld %hd %hd %hd %hd\n",
	       &(inventory_init[count].p1),
	       &(inventory_init[count].tohit),
	       &(inventory_init[count].todam),
	       &(inventory_init[count].ac),
	       &(inventory_init[count].toac)  );
	
	fgets(in_line, sizeof(in_line), infile);
	in_line[strlen(in_line)-1] = 0;
	strcpy(inventory_init[count].damage, in_line);

	fgets(in_line, sizeof(in_line), infile);
	inventory_init[count].tchar = in_line[0];

      }
    }

    fclose(infile);
  }

  if (!return_value) {
    prt ("ERROR: A severe error occurred in read_inventory_list.",1,1);
    prt ("Please notify the BOSS Master...",2,1);
    pause_line(24);
  }

  return return_value;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
boolean read_data()
{
  boolean  return_value = true;

  return_value &= read_creatures();
  return_value &= read_objects();
  return_value &= read_inventory_list();

  return return_value;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
boolean get_skill_data(integer number)
{
  /*
    {Loads skill data from file into skill_list array. -RLG}
    {Seems like a better way to handle the skill lists. }
    {But geez did this take a lot of work to implement!!! }
    {I was warned.. 
    
    { I should have listened to what my mother used to say. }
    { What did she say? }
    { I don't know.  I wasn't listening. }
  */

  integer  pos;
  vtype    in_line;
  FILE    *infile;
  integer  i1, i2, i3, i4;
  boolean  return_value = true;
        
  infile = priv_fopen( SKILLS, "r");
  if (infile == NULL) {
    return_value = false;
  } else {

    do {
      /*
       * Throw away lines until we find one that starts with "**"
       * The line after that has the number of the skill section.
       */
      do {
	fgets(in_line, sizeof(in_line), infile);
      } while ( strncmp(in_line,"**",2) );
      
      fgets(in_line, sizeof(in_line), infile);
      sscanf(in_line, "%ld\n", &pos);
    } while (pos != number);

    for (pos = 1; pos <= MAX_SKILLS; pos++) {
      //with skill_list[pos]. do;
      fgets(in_line, sizeof(in_line), infile);
      strtrim( in_line );
      strcpy(skill_list[pos].sname, in_line);

      fgets(in_line, sizeof(in_line), infile);
      sscanf(in_line,"%ld %ld %ld %ld", &i1, &i2, &i3, &i4);
      skill_list[pos].slevel  = i1;
      skill_list[pos].smana   = i2;
      skill_list[pos].sexp    = i3;
      skill_list[pos].sfail   = i4;
      skill_list[pos].learned = false;
    }

    fclose(infile);
  }

  if (!return_value) {
    prt ("ERROR: A severe error occurred in get_skill_data.",1,1);
    prt ("Please notify the BOSS Master...",2,1);
    pause_line(24);
  }

  return return_value;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

/* END FILE datafiles.c */
