/* display.c */
/*
  {This Include file contains all the inventory display routines.
  Many of which have been extremely modified by  -RLG}
*/

#include "boss.h"

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
char raoul_label(integer position)
{
  /*{Returns the character label (a-Z) of an integer.}*/

  return alpha_set[position-1];
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
char cur_char1(integer item_val)
{
  /*
    { Returns a '*' for cursed items, a ')' for normal ones
    NOTE: '*' returned only if item has been identified.}
  */
  char return_value;
  
  //with inventory[item_val]. do;
  if (uand(0x80000000,inventory[item_val].flags) == 0) {
    return_value = ')';    /*{ Not cursed.}*/
  } else if (strchr(inventory[item_val].name,'|') != NULL) {
    return_value = ')';    /*{ Cursed, but not identified }*/
  } else if (strchr(inventory[item_val].name,'^') != NULL) {
    return_value = ')';    /*{ Cursed, but not identified }*/
  } else {
    return_value = '*';    /*{ Cursed and identified.}*/
  }

  return return_value;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
char cur_char2(integer item_val)
{
  /*{ Returns a '*' for cursed items, a ')' for normal ones }*/

  //with equipment[item_val]. do;

  char return_value;

  if (uand(0x80000000,equipment[item_val].flags) == 0) {
    return_value = ')'; /*{ Not cursed.}*/
  } else {
    return_value = '*'; /*{ Cursed.}*/
  }

  return return_value;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
boolean get_item(integer *com_val, char *pmt, boolean *redraw,
		 integer r1, integer r2 )
{
  /*{ Get the ID of an item and return the CTR value of it }*/

  char     command;
  integer  cur_top;
  vtype    out_val;
  boolean  exit_flag = false;
  boolean  return_value = false;
  
  cur_top = r1;
  *redraw = false;
  
  if (inven_ctr > 0) {

    sprintf(out_val," (%c-%c) <space for list, <esc> to exit.  %s",
	    raoul_label(r1),raoul_label(r2),pmt);
    prt(out_val,1,1);

    do {
      command  = inkey();
      *com_val = command;
      switch (*com_val) {

      case 0: case 3: case 13: case 25: case 26: case 27:  
	/* { Exit from module } */
	exit_flag  = true;
	reset_flag = true;
	break;

      case 32 :  
	if (! *redraw) {
	  inven_command('?',cur_top,r2);
	  *redraw = true;
	} else {
	  if (cur_top + INVEN_PER_PAGE <= r2) {
	    cur_top += INVEN_PER_PAGE;
	  } else {
	    cur_top  = r1;
	  }
	  inven_command('?',cur_top,r2);
	}
	break;

      default:
	*com_val = pindex(alpha_set,command);
	if ((*com_val >= r1) && (*com_val <= r2)) {
	  exit_flag    = true;
	  return_value = true;
	}
	break;
      }
    } while (!exit_flag);
    erase_line(msg_line,msg_line);
  } else {
    msg_print("You are not carrying anything.");
  }
  return return_value;
}
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
static void ic__show_inven(integer *scr_state, integer r1, integer r2)  
{
  /*{Much Improved by RLG}*/
  /*
    { Displays inventory items from r1 to r2 }
    {r1 can be cur_top or start of range,  r2 can be inven_ctr or end of range}
  */

  integer   i1;
  vtype     tmp_val,out_val;

  if (r1 > 0) {      /*{ R1 = 0 dummy call }*/
    clear_from(2);
    *scr_state = 1;  /*{ Set state to 1 }*/

    if (r2-r1 > INVEN_PER_PAGE-1) {
      r2 = r1 + INVEN_PER_PAGE-1;
    }

    for (i1 = 0; i1 <= (r2-r1); i1++) {
      objdes(tmp_val,r1+i1,true,'i');
      sprintf(out_val,"%c%c %s",raoul_label(r1+i1),cur_char1(r1+i1),tmp_val);
      prt(out_val,i1+2,1);
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void ic__show_equip(integer *scr_state, integer r1)
{
  /*{ Displays equipment items from r1 to end }*/

  integer  i1,i2;
  vtype    prt2,out_val;
  char    *prt1;

  if (r1 > equip_ctr) {   /*{ Last item gone }*/
    prt("",equip_ctr+3,1);
  } else if (r1 > 0) {   /*{ R1 = 0 dummy call }*/
    i2 = 0;
    for (i1 = 23; i1 <= EQUIP_MAX; i1++) { /*{ Range of equipment }*/
      //with equipment[i1]. do;
      if (equipment[i1].tval > 0) {
	i2++;
	if (i2 >= r1) { /*{ Display only given range }*/
	  switch (i1) { /*{ Get position }*/
	  case 23 :      prt1 = " You are wielding   : "; break;
	  case 24 :      prt1 = " Worn on head       : "; break;
	  case 25 :      prt1 = " Worn around neck   : "; break;
	  case 26 :      prt1 = " Worn on body       : "; break;
	  case 27 :      prt1 = " Worn on arm        : "; break;
	  case 28 :      prt1 = " Worn on hands      : "; break;
	  case 29 :      prt1 = " Worn on right hand : "; break;
	  case 30 :      prt1 = " Worn on left hand  : "; break;
	  case 31 :      prt1 = " Worn on feet       : "; break;
	  case 32 :      prt1 = " Worn about body    : "; break;
	  case 33 :      prt1 = " Light source       : "; break;
	  case 34 :      prt1 = " Secondary weapon   : "; break;
	  case 35 :      prt1 = " Computer           : "; break;
	  default:       prt1 = " Unknown value: ";     ; break;
	  }
	  objdes(prt2,i1,true,'e');
	  sprintf(out_val,"%c%c%s%s",(int)(i2+96),cur_char2(i1),prt1,prt2);
	  prt(out_val,i2+2,1);
	}
      }
    };
    prt("",i2+3,1);   /*{ Clear last line }*/
    *scr_state = 2;   /*{ Set state of screen }*/
  }
};
//////////////////////////////////////////////////////////////////////
static void ic__count_inventory(integer *scr_state)
{
  /*{Summary of player's inventory displayed to the screen} */
  
  integer    i1,i2,i3,row,num;
  
  obj_set    weapons = {20,21,22,23};
  obj_set    armor   = {30,31,32,33,34,35,36};
  obj_set    tossing = {82,84};
  obj_set    ammo    = {10,11,12};
  obj_set    foods   = {80};
  obj_set    potions = {75,76};
  obj_set    floppy  = {70,71};
  obj_set    ray_gun = {65};
  obj_set    wierd   = {55};
  obj_set    smalls  = {40,45};

  num = 0;
  prt("Inventory Summary",16,22);
  row = 17;

  if (find_range(weapons,&i2,&i3)) {
    num = i3-i2+1;
    row = row+1;
    prt_num("Weapons         : ",num,row,7);
  }
  
  if (find_range(armor,&i2,&i3)) {
    num = i3-i2+1;
    row = row+1;
    prt_num("Armor           : ",num,row,7);
  }
  
  if (find_range(tossing,&i2,&i3)) {
    num = 0;
    for (i1 = i2; i1 <= i3; i1++) {
      num = num + inventory[i1].number;
    }
    row = row+1;
    prt_num("Thrown/Grenades : ",num,row,7);
  }
  
  if (find_range(ammo,&i2,&i3)) {
    num = 0;
    for (i1 = i2; i1 <= i3; i1++) {
      num = num + inventory[i1].number;
    }
    row = row+1;
    prt_num("Rounds of Ammo  : ",num,row,7);
  }
  
  if (find_range(foods,&i2,&i3)) {
    num = 0;
    for (i1 = i2; i1 <= i3; i1++) {
      num = num + inventory[i1].number;
    }
    row = row+1;
    prt_num("Food Items      : ",num,row,7);
  }
  
  row = 17;
  
  if (find_range(potions,&i2,&i3)) {
    num = 0;
    for (i1 = i2; i1 <= i3; i1++) {
      num = num + inventory[i1].number;
    }
    row = row+1;
    prt_num("Potions         : ",num,row,35);
  }
  
  if (find_range(floppy,&i2,&i3)) {
    num = 0;
    for (i1 = i2; i1 <= i3; i1++) {
      num = num + inventory[i1].number;
    }
    row = row+1;
    prt_num("Floppy Disks    : ",num,row,35);
  }
  
  if (find_range(ray_gun,&i2,&i3)) {
    num = i3-i2+1;
    row = row+1;
    prt_num("Ray Guns        : ",num,row,35);
  }
  
  if (find_range(wierd,&i2,&i3)) {
    num = i3-i2+1;
    row = row+1;
    prt_num("Wierd Devices   : ",num,row,35);
  }
  
  if (find_range(smalls,&i2,&i3)) {
    num = i3-i2+1;
    row = row+1;
    prt_num("Amulets/Rings   : ",num,row,35);
  }
};
//////////////////////////////////////////////////////////////////////
static void ic__clean(integer *scr_state, integer *cur_top)
{
  /*{ Displays inventory items and cleans item from list }*/

  integer   com_val;
  vtype     out_val, out_val2;
  char      command;
  boolean   exit_flag,test_flag;

  exit_flag = false;

  do {
    test_flag = false;
    do {
      sprintf(out_val,
	      "( <space> for next page, <esc> to exit ) "
	      "Lose which item ?");
      prt(out_val,1,1);

      command = inkey();
      switch (command) {
      case 0: case 3: case 13: case 25: case 26: case 27:  
	test_flag = true;
	exit_flag = true;
	break;

      case 32 :  
	if (*cur_top+INVEN_PER_PAGE <= inven_ctr) {
	  *cur_top = *cur_top + INVEN_PER_PAGE;
	} else {
	  *cur_top = 1;
	}
	ic__show_inven(scr_state,*cur_top,inven_ctr);
	break;

      default:
	com_val = pindex(alpha_set,command);
	if (com_val >= 1 && com_val <= inven_ctr) {
	  objdes(out_val,com_val,true,'i');
	  sprintf(out_val2,"Destroyed %s", out_val);
	  inven_destroy(com_val);
	  ic__show_inven(scr_state,*cur_top,inven_ctr);
	  msg_print(out_val2);
	  msg_print("");
	  test_flag = true;
	}
	break;
      }

    } while (!test_flag);
  } while (!exit_flag);

  erase_line(msg_line,msg_line);
};
//////////////////////////////////////////////////////////////////////
static integer ic__remove(integer *scr_state, integer item_val)
{
    /*{ Remove item from equipment list }*/
  integer        i1,i2,typ;
  vtype          out_val,prt2;
  char          *prt1;
  boolean        flag;
  treasure_type  item_removed;

  i1 = 0;
  flag = false;
  if (item_val == -1) {
    item_removed = temporary_slot;
  } else {
    item_removed = equipment[item_val];
  }
  
  typ = item_removed.tval;
  
  do {
    i1++;
    //with inventory[i1]. do;
    if (typ > inventory[i1].tval) {
      for (i2 = inven_ctr; i2 >= i1; i2--) {
	inventory[i2+1] = inventory[i2];
      }
      inventory[i1] = item_removed;
      inven_ctr++;
      equip_ctr--;
      flag = true;
    }
  } while (!flag);
  
  switch (typ) {
  case 10: case 11: case 12: case 20:
  case 21: case 22: case 23: case 25: 
    prt1  = "Was wielding ";
    break;
  case 15:
    prt1  = "Light source was ";
    break;
  default:
    prt1  = "Was wearing ";
    break;
  }
  
  objdes(prt2,i1,true,'i');
  sprintf(out_val,"%s%s (%c)",prt1,prt2,raoul_label(i1));
  msg_print(out_val);
  
  if (item_val == -1) {
    temporary_slot = blank_treasure;
  } else {
    equipment[item_val] = blank_treasure;          
  }
  
  if (item_val != EQUIP_MAX) {  /*{ For secondary weapon}*/
    py_bonuses(&inventory[i1],-1);
  }
  
  return i1;
};
//////////////////////////////////////////////////////////////////////
static void ic__unwear(integer *scr_state)
{
    /*{ Unwear routine, remove a piece of equipment }*/

  integer   i1,i2,com_val;
  boolean   exit_flag,test_flag;
  char      command;
  vtype     out_val;

  if (*scr_state == 1) {
    clear_from(1);
    ic__show_equip(scr_state,1);
  }

  exit_flag = false;

  do {
    sprintf(out_val,"(a-%c, <space> for equipment list, <esc> to exit) "
	    "Take off which one ?", (int)(equip_ctr+96));
      test_flag = false;
    msg_print(out_val);

    do {
      command = inkey();
      
      switch (command) {
      case 0: case 3: case 25: case 26: case 27:  
	test_flag = true;
	exit_flag = true;
	break;
	
      case 32 :      
	clear_from(2);
	ic__show_equip(scr_state,1);
	break;
	
      default:
	com_val = command - 96;
	if ((com_val >= 1) &&
	    (com_val <= equip_ctr)) {
	  test_flag = true;
	}
	break;
      }

    } while (!test_flag);

    if (!exit_flag) {
      reset_flag = false; /*{ Player turn }*/
      i1 = 0;
      i2 = 22;
      
      do {
	i2 = i2 + 1;
	if (equipment[i2].tval > 0) {
	  i1 = i1 + 1;
	}
      } while (i1 != com_val);
      
      if (uand(0x80000000,equipment[i2].flags) != 0) {
	msg_print("Hmmm, it seems to be really stuck...");
	com_val = 0;
      } else {
	ic__remove(scr_state,i2);
      }
    }
    
    if (*scr_state == 0) {
      exit_flag = true;
    } else if (equip_ctr == 0) {
      exit_flag = true;
    } else if (inven_ctr > 21) {
      exit_flag = true;
    } else if (!exit_flag) {
      ic__show_equip(scr_state,com_val);
    }
    
  } while (!exit_flag);
  
  if (*scr_state != 0) {
    if (equip_ctr == 0) {
      clear_from(1);
    } else {
      prt("You are currently using -",1,1);
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void ic__wear(integer *scr_state, integer *cur_top)
{
  /*{ Wear routine, wear or wield an item }*/

  integer        com_val,i1,i2,i3,tmp;
  vtype          out_val,out_val2,prt2;
  char          *prt1, command;
  treasure_type  unwear_obj;
  boolean        exit_flag,test_flag;

  exit_flag = false;
  
  do {
    test_flag = false;
    
    do {
      sprintf(out_val,
	      "( <space> for next page, <esc> to exit ) "
	      "Wear/Wield which item?");
      prt(out_val,1,1);

      command = inkey();
      com_val = command;
      switch (com_val) {
      case 0: case 3: case 13: case 25: case 26: case 27:
	/*{ Exit from module }*/
	test_flag = true;
	exit_flag = true;
	break;

      case 32 :  
	if (*scr_state != 1) {  /*{set scr_state to 1}*/
	  ic__show_inven(scr_state,*cur_top,inven_ctr);
	} else {
	  if (*cur_top+INVEN_PER_PAGE <= inven_ctr) {
	    *cur_top = *cur_top + INVEN_PER_PAGE;
	  } else {
	    *cur_top = 1;
	  }
	  ic__show_inven(scr_state,*cur_top,inven_ctr);
	}
	break;
                
      default:
	com_val = pindex(alpha_set,command);
	if (com_val >= 1 && com_val <= inven_ctr) {
	  test_flag = true;
	}
	break;
      } /* end switch */
      
    } while (!test_flag);


    if (!exit_flag) { /*{ Main logic for wearing }*/
      reset_flag = false; /*{ Player turn }*/
      test_flag  = true;

      switch (inventory[com_val].tval) { /*{ Slot for equipment }*/
      case 15:          i1 = 33; break;
      case 19: case 20: case 21: 
      case 23: case 25: i1 = 23; break;
      case 29:          i1 = 35; break;
      case 30:          i1 = 31; break;
      case 31:          i1 = 28; break;
      case 32:          i1 = 32; break;
      case 33:          i1 = 24; break;
      case 34:          i1 = 27; break;
      case 35: case 36: i1 = 26; break;
      case 40:          i1 = 25; break;
      case 45: 
	if (equipment[29].tval == 0) {   /*{ Rings }*/
	  i1 = 29;
	} else {
	  i1 = 30;
	}
	break;
      default:
	msg_print("Sorry, you can't wear or wield that.");
	msg_print("");
	test_flag = false;
	com_val = 0;
	break;
      } /* end switch */

      if (test_flag) {
	if (equipment[i1].tval > 0) {
	  if (uand(0x80000000,equipment[i1].flags) != 0) {
	    objdes(out_val2,i1,false,'e');
	    sprintf(out_val, "The %s you are ", out_val2);
	    switch (i1) {
	    case 23 :
	      strcat(out_val, "wielding ");
	      break;
	    default :
	      strcat(out_val, "wearing ");
	      break;
	    }
	    strcat(out_val, "appears to be cursed.");
	    msg_print(out_val);
	    msg_print("");
	    test_flag = false;
	    com_val = 0;
	  } else if (inven_ctr > INVEN_MAX-1) {
	    if (inventory[com_val].number > 1) {
	      if (inventory[com_val].subval < 512) {
		msg_print("You will have to drop something first.");
		test_flag = false;
		com_val = 0;
	      }
	    }
	  }
	}
      }

      if (test_flag) {
	unwear_obj = equipment[i1];
	equipment[i1] = inventory[com_val];
	//with equipment[i1]. do;
	/*{Fix for weight}*/
	inven_weight += equipment[i1].weight * equipment[i1].number;
	if ((equipment[i1].subval >= 255) && (equipment[i1].subval < 512)) {
	  equipment[i1].number = 1;
	  /*{Fix for flashlights}*/
	  if (i1 == 33)  { equipment[i1].subval -= 255; }
	}
	
	inven_destroy(com_val); /*{ Subtracts weight }*/
	if (*scr_state != 0) {
	  ic__show_inven(scr_state,*cur_top,inven_ctr);
	}
	equip_ctr = equip_ctr + 1;
	py_bonuses(&equipment[i1],1);
	if (unwear_obj.tval > 0) {
	  temporary_slot = unwear_obj;
	  tmp = ic__remove(scr_state,-1);
	  if (tmp < com_val) {
	    com_val = tmp;
	  }
	  if (*scr_state != 0) {
	    ic__show_inven(scr_state,*cur_top,inven_ctr);
	  }
	}
	switch (i1) {
	case 23 : prt1 = "You are wielding ";     break;
	case 33 : prt1 = "Your light source is "; break;
	default : prt1 = "You are wearing ";      break;
	}
	objdes(prt2,i1,true,'e');
	i2 = 0;
	i3 = 22;
	do {   /*{ Get the right letter of equipment }*/
	  i3 = i3 + 1;
	  if (equipment[i3].tval > 0) {
	    i2 = i2 + 1;
	  }
	} while (i3 != i1);
	sprintf(out_val, "%s%s (%c%c",prt1,prt2,(int)(i2+96),cur_char2(i1));
	msg_print(out_val);
	msg_print("");
      } /* if test_flag */
      
    } /* if !exit_flag */
    
    if (*scr_state == 0) {
      exit_flag = true;
    } else if (inven_ctr == 0) {
      exit_flag = true;
    }
    
  } while (!exit_flag);
  
  if (*scr_state != 0) {
    prt("You are currently carrying -",1,1);
  }
};
//////////////////////////////////////////////////////////////////////
static void ic__switch_weapon(integer *scr_state)
{
    /*{ Switch primary and secondary weapons }*/

  vtype          prt1,prt2,out_val;
  treasure_type  tmp_obj;

  if (uand(0x80000000,equipment[23].flags) != 0) {
    objdes(prt1,23,false,'e');
    sprintf(out_val, "The %s you are wielding is really stuck to you.",prt1);
    msg_print(out_val);
    msg_print("");
  } else {
    /*{ Switch weapons }*/
    reset_flag = false;
    tmp_obj = equipment[34];
    equipment[34] = equipment[23];
    equipment[23] = tmp_obj;
    py_bonuses(&equipment[34],-1);  /*{ Subtract bonuses }*/
    py_bonuses(&equipment[23],1);   /*{ Add bonuses }*/

    if (equipment[23].tval > 0) {
      objdes(prt2,23,true,'e');
      sprintf(out_val,"Primary weapon: %s", prt2);
      msg_print(out_val);
      msg_print("");
    }

    if (equipment[34].tval > 0) {
      objdes(prt2,34,true,'e');
      sprintf(out_val, "Secondary weapon: %s", prt2);
      msg_print(out_val);
      msg_print("");
    }
  }

  if (*scr_state != 0) {
    msg_print("");
    clear_from(1);
    prt("You are currently using -",1,1);
    ic__show_equip(scr_state,1);
  }
};
//////////////////////////////////////////////////////////////////////
boolean inven_command(char command, integer r1, integer r2)
{
  /*{ Main logic for INVEN_COMMAND }*/
  
  integer   scr_state;
  integer   cur_top;
  integer   exit_flag, test_flag;
  boolean   return_value = false;
  
  exit_flag = false;
  scr_state = 0;
  cur_top   = 1;
  
  do {
    switch (command) {
    case 'i' :    /*{ Inventory }*/
      if (inven_ctr == 0) {
	msg_print("You are not carrying anything.");
      } else if (scr_state != 1) {
	/*  { Sets scr_state to 1 }*/
	erase_line(1,1);
	prt("You are currently carrying -",1,1);
	ic__show_inven(&scr_state,cur_top,inven_ctr);
      }
      break;
      
    case ' ' : 
      if (cur_top+INVEN_PER_PAGE <= inven_ctr) {
	cur_top += INVEN_PER_PAGE;
      } else {
	cur_top = 1;
      }
      ic__show_inven(&scr_state,cur_top,inven_ctr);
      break;
      
    case 'c' :  /*{ Clean }*/
      if (inven_ctr == 0) {
	msg_print("You are not carrying anything.");
      } else {
	ic__clean(&scr_state, &cur_top);
      }
      break;
      
    case 'e' :    /*{ Equipment }*/
      if (equip_ctr == 0) {
	msg_print("You are not using any equipment.");
      } else if (scr_state != 2) {
	/*{ Sets scr_state to 2 }*/
	clear_from(1);
	prt("You are currently using -",1,1);
	ic__show_equip(&scr_state,1);
      }
      break;
      
    case 't' :    /*{ Take off }*/
      if (equip_ctr == 0) {
	msg_print("You are not using any equipment.");
      } else if (inven_ctr > INVEN_MAX-1) {
	msg_print("You will have to drop something first.");
      } else {
	ic__unwear(&scr_state);   /*{ May set scr_state to 2 }*/
      }
      break;
      
    case 'w' :    /*{ Wear/wield }*/
      if (inven_ctr == 0) {
	msg_print("You are not carrying anything.");
      } else {
	ic__wear(&scr_state, &cur_top);  /*{ May set scr_state to 1 }*/
      }
      break;
      
    case 'x' :  /* exchange primary and secondary weapons */
      if (equipment[23].tval != 0) {
	ic__switch_weapon(&scr_state);
      } else if (equipment[34].tval != 0) {
	ic__switch_weapon(&scr_state);
      } else {
	msg_print("But you are wielding no weapons.");
      }
      break;
      
      /*{ Special function for other routines }*/
    case '?' :  /*{ Displays part inven, returns}*/
      ic__show_inven(&scr_state,r1,r2);
      scr_state = 0;  /*{ Clear screen state }*/
      break;
      
      /*{ Nonsense command }*/
    default:
      break;
      
    }
    
    if (scr_state > 0) {
      prt("<e>quip, <i>inven, <t>ake-off, <w>ear/wield, "
	  "e<x>change, <c>lean, ^Z to exit.",24,2);
      test_flag = false;
      if (scr_state == 1) {
	ic__count_inventory(&scr_state);
      }
      do {
	command = inkey();
	switch (command) {
	case 0: case 3: case 25: case 26: case 27:  
	  /*{ Exit from module }*/
	  exit_flag = true;
	  test_flag = true;
	  break;
	default:
	  switch (command) {  /*{ Module commands }*/
	  case ' ' :
	  case 'c' :
	  case 'e' :
	  case 'i' :
	  case 't' :
	  case 'w' :
	  case 'x' : 
	    test_flag = true;
	    break;
	  case '?' : ;  /*{ Trap special feature}*/
	    break;
	  default: ;    /*{ Nonsense command }*/
	    break;
	  }
	  break;
	}
      } while (!test_flag);
      prt("",24,1);
      
    } else {
      exit_flag = true;
    }
  } while (!exit_flag);
    
  if (scr_state > 0) {   /*{ If true, must redraw screen }*/
    return_value = true;
  }

  return return_value;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

/* END FILE display.c */
