/* files.c */
/**/

#include "boss.h"
#include "master.h"
#include <sys/file.h>      /* for flock     */

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
static
boolean user_vs_file(vtype boss_file)
{
  /*Check username to users listed in a file. If in file returns true. -RLG*/
  
  boolean     return_value = false;
  FILE       *file1 = NULL;
  vtype       in_line;

  file1 = priv_fopen( boss_file, "r");

  if (file1 == NULL) {
    printf ("ERROR: A severe error occurred in ID Section.\n");
  } else {
    while (!feof(file1)) {
      in_line[0] = 0;
      fgets(in_line, sizeof(in_line), file1);
      strtrim(in_line);
      if (!strcmp(username, in_line)) {
	return_value = true;
	break;
      }
    }
    fclose(file1);
  }

  return return_value;
};

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void look_at_userid()
{
  validuser = user_vs_file(BOSS_WIZ);
  scumuser  = user_vs_file(BOSS_LOS);
  putzuser  = user_vs_file(BOSS_ANNOY);
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

FILE *priv_fopen( char *path, char *mode)
{
  FILE *f1;
  /* call fopen bracketed by priv_switch calls */

  priv_switch(1);

  f1 = (FILE *)fopen(path, mode);

  priv_switch(0);

  return f1;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

boolean intro_do_hours_file (boolean already_exiting, char * the_file)
{
//      { Attempt to read hours.dat.  If it does not exist,     }
//      { then create a standard one.                           }

  FILE     *file1;
  vtype    in_line;
  integer  i1;
  boolean  exit_flag = false;

  file1 = priv_fopen(the_file, "r");
  if (file1 != NULL)
    {
      while (!feof(file1)) {
        in_line[0] = 0;
        fgets(in_line, sizeof(in_line), file1);

        chomp(in_line);
        if (strlen(in_line) > 3) {
          if        (!(strncmp("SUN:", in_line,4))) {
            strncpy(days[0], in_line, sizeof(days[0]));
          } else if (!(strncmp("MON:", in_line,4))) {
            strncpy(days[1], in_line, sizeof(days[1]));
          } else if (!(strncmp("TUE:", in_line,4))) {
            strncpy(days[2], in_line, sizeof(days[2]));
          } else if (!(strncmp("WED:", in_line,4))) {
            strncpy(days[3], in_line, sizeof(days[3]));
          } else if (!(strncmp("THR:", in_line,4))) {
            strncpy(days[4], in_line, sizeof(days[4]));
          } else if (!(strncmp("FRI:", in_line,4))) {
            strncpy(days[5], in_line, sizeof(days[5]));
          } else if (!(strncmp("SAT:", in_line,4))) {
            strncpy(days[6], in_line, sizeof(days[6]));
          }
        } /* end if */
        
      } /* end while */
      fclose(file1);

    } else {
      /* try to create the default hours.dat file */
      priv_switch(0);
      file1 = (FILE *)fopen(the_file,"w");
      if (file1 != NULL)
        {
          fprintf(file1,"    Boss operating hours are:\n");
          fprintf(file1,"    |    AM     |    PM     |\n");
          fprintf(file1,"    1         111         111\n");
          fprintf(file1,"    2123456789012123456789012\n");
          for (i1 = 0; i1 < 7; i1++) {
            fprintf(file1,"%s\n",days[i1]);
          }
          fprintf(file1,"       (X=Open; .=Closed)\n");
          fclose(file1);
          printf("Created %s\n",the_file);
          exit_flag = true;
        } else {
          printf("Error in creating %s for output.\n\r",the_file);
          exit;
        } /* end if file1 (writing) */

    } /* end if file1 (reading) */  

  return (exit_flag || already_exiting);
}; /* end intro_do_hours_file */

//////////////////////////////////////////////////////////////////////

boolean intro_do_msg_file(boolean already_exiting, char * the_file, 
			  boolean write_to_screen)
{
//      { Print the introduction message, news, ect...          }

  FILE     *file1;
  int      i1, i2;
  vtype    in_line;
  boolean  exit_flag = false;

  file1 = priv_fopen(the_file,"r");
  if (file1 != NULL)
    {
      if (!already_exiting  &&  write_to_screen) {
        /* print out the BOSS_MOR file, move to upper left */
	clear_rc(1,1);
        for (i1 = 1; !feof(file1); i1++) {
          in_line[0] = 0;
          fgets(in_line, sizeof(in_line), file1);
	  if ((i2 = strlen(in_line)) > 0) {
	    in_line[i2-1] = 0; 
	    put_buffer(in_line, i1, 1);
	  }
        }
        pause_exit(24,0);
      }
      fclose(file1);
      
    } else {
      /* create a new BOSS_MOR file */
      file1 = (FILE *)fopen(the_file,"w");
      if (file1 == NULL)
        {
          printf("Error opening %s for writing\n\r",the_file);
          exit_game();
        } else {
          
fprintf(file1,"                         *********************\n");
fprintf(file1,"                         **     BOSS %4.2f   **\n",CUR_VERSION);
fprintf(file1,"                         *********************\n");
fprintf(file1,"                   COPYRIGHT (c) Robert Alan Koeneke\n");
fprintf(file1,"\n");
fprintf(file1,"Programers : Robert Alan Koeneke / University of Oklahoma\n");
fprintf(file1,"             Jimmey Wayne Todd   / University of Oklahoma\n");
fprintf(file1,"\n");
fprintf(file1,"Modified by: Bill Palter         / State University of New York\n");
fprintf(file1,"             Nick B Triantos     / State University of New York\n");
fprintf(file1,"\n");
fprintf(file1,"BOSS Modifications by: Robert Gulledge / University of Washington\n");
fprintf(file1,"                       jason black     / University of Washington\n");
fprintf(file1,"\n");
fprintf(file1,"Linux port by:         Stephen Kertes, 1999-2000.\n");
fprintf(file1,"\n");
fprintf(file1,"\n");
fprintf(file1,"             Check NewsInc in the office building for new features!.\n");
fprintf(file1,"\n");


        printf("Created %s\n\r", the_file);
        fclose(file1);
        exit_flag = true;
        } /* end if file1 (writing) */
      
    } /* end if file1 (reading) */

  return (exit_flag || already_exiting);
}; /* end intro_do_msg_file */

//////////////////////////////////////////////////////////////////////
boolean intro_do_bus_file(boolean already_exiting, char * the_file)
{
  FILE     *file1;
  boolean  exit_flag = false;

  file1 = priv_fopen(the_file,"r");
  if (file1 != NULL)
    {
      /* file already exists and can be opened */
      fclose(file1);
    } else {
      /* create a new file */
      file1 = (FILE *)fopen(the_file,"w");
      if (file1 == NULL) {
        printf("Error creating %s\n\r", the_file);
        exit_game();
      } else {
        printf("Created %s\n\r", the_file);

fprintf(file1,"\n");
fprintf(file1,"\n");
fprintf(file1,"\n");
fprintf(file1,"\n");
fprintf(file1,"\n");
fprintf(file1,"\n");
fprintf(file1,"\n");
fprintf(file1,"\n");
fprintf(file1,"\n");
fprintf(file1,"               +----------------------+\n");
fprintf(file1,"               | [ ] [@] [ ] [ ] [ ] & \\\n");
fprintf(file1,"               |^^^^^^^^^^^^^^^^^^^^^^^ ]\n");
fprintf(file1,"           ****`-(0)(0)------------(0)--'\n");
fprintf(file1,"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n");
fprintf(file1,"\n");
fprintf(file1,"\n");
fprintf(file1,"                                                  +---------------------+\n");
fprintf(file1,"                                                  |                     |\n");
fprintf(file1,"                                                  |       ------>       |\n");
fprintf(file1,"                                                  +----------+----------+\n");
fprintf(file1,"                                                             |\n");
fprintf(file1,"                                                             |\n");
fprintf(file1,"                                                             |\n");
fprintf(file1,"\n");
fprintf(file1,"\n");
fprintf(file1,"\n");
fprintf(file1,"\n");
fprintf(file1,"\n");
fprintf(file1,"\n");
fprintf(file1,"\n");

        fclose(file1);
        exit_flag = true;
      } /* end if file1 (writing) */
    } /* end if file1 (reading) */

  return (exit_flag || already_exiting);

}; /* end intro_do_bus_file */
//////////////////////////////////////////////////////////////////////
boolean intro_do_quote_file(boolean already_exiting, char * the_file)
{
  FILE     *file1;
  boolean  exit_flag = false;

  file1 = priv_fopen(the_file,"r");
  if (file1 != NULL)
    {
      /* file already exists and can be opened */
      fclose(file1);
    } else {
      /* create a new file */
      file1 = (FILE *)fopen(the_file,"w");
      if (file1 == NULL) {
        printf("Error creating %s\n\r", the_file);
        exit_game();
      } else {
        printf("Created %s\n\r", the_file);

fprintf(file1,"\n");
fprintf(file1,"12\n");
fprintf(file1,"**\n");
fprintf(file1,"You see written on the wall the words \"DON'T PANIC!\" in big red letters.\n");
fprintf(file1,"**\n");
fprintf(file1,"This must be Thursday.  You never could get the hang of Thursdays.\n");
fprintf(file1,"**\n");
fprintf(file1,"That was a pretty hoopy bit of thinking Earthman.\n");
fprintf(file1,"**\n");
fprintf(file1,"We will return to normality as soon as we figure out\n");
fprintf(file1,"what's normal anyway.\n");
fprintf(file1,"**\n");
fprintf(file1,"We have reached normality.\n");
fprintf(file1,"Anything you can't cope with now is therefore your own problem.\n");
fprintf(file1,"**\n");
fprintf(file1,"What are you supposed to do with a manically depressed robot?\n");
fprintf(file1,"You think you've got problems?\n");
fprintf(file1,"What are you supposed to do if you _are_ a manically depressed robot?\n");
fprintf(file1,"**\n");
fprintf(file1,"_mostly_ harmless...\n");
fprintf(file1,"**\n");
fprintf(file1,"A voice booms down from above...\n");
fprintf(file1,"\"Be excellent to each other!\"\n");
fprintf(file1,"**\n");
fprintf(file1,"A voice booms down from above...\n");
fprintf(file1,"\"Party on, dudes!\"\n");
fprintf(file1,"**\n");
fprintf(file1,"A voice booms down from above...\n");
fprintf(file1,"\"Don't forget to wind your watch!!\"\n");
fprintf(file1,"**\n");
fprintf(file1,"A voice booms down from above...\n");
fprintf(file1,"\"Yes way, Ted!\"\n");
fprintf(file1,"**\n");
fprintf(file1,"You suddenly ask yourself...\n");
fprintf(file1,"\"What is the mean air speed of an african swallow\n");
fprintf(file1,"carrying a five point coconut?\"\n");
fprintf(file1,"**\n");

        fclose(file1);
        exit_flag = true;
      } /* end if file1 (writing) */
    } /* end if file1 (reading) */

  return (exit_flag || already_exiting);

}; /* end intro_do_quote_file */
//////////////////////////////////////////////////////////////////////
boolean intro_ensure_file_exists(boolean already_exiting, char * the_file)
{
  FILE     *file1;
  boolean  exit_flag = false;

  file1 = priv_fopen(the_file,"r");
  if (file1 != NULL)
    {
      /* file already exists and can be opened */
      fclose(file1);
    } else {
      /* create a new file */
      file1 = (FILE *)fopen(the_file,"w");
      if (file1 == NULL) {
        printf("Error creating %s\n\r", the_file);
        exit_game();
      } else {
        printf("Created %s\n\r", the_file);
        fclose(file1);
        exit_flag = true;
      } /* end if file1 (writing) */
    } /* end if file1 (reading) */

  return (exit_flag || already_exiting);
}; /* end intro_ensure_file_exists */

//////////////////////////////////////////////////////////////////////
void print_version_info()
{
  printf("\n\r");
  printf("Linux Boss Version %3.2f, patch level %d\n\r",
	 CUR_VERSION, PATCH_LEVEL);
  printf("\n\r");
}; /* end print_version_info */
//////////////////////////////////////////////////////////////////////
void intro_qualifier_help()
{
  printf("\n\r");
  printf("Invalid Boss option!  Valid qualifiers are:\n\r");
  printf("  -f file    Load saved character in file.\n\r");
  printf("  -s         List top 20 high scores.\n\r");
  printf("  -t num     List <num> high scores after death or for -s.\n\r");
  printf("  -Q         Quit after checking for data files.\n\r");
  printf("  -V         Print version info.\n\r");

//  check_pswd("doublespeak",true); /*XXXX get a passwd from command line... */
  if (wizard)
    {
      printf("\n\r\n\rWizard commands:\n\r");
      printf("  -Wpassword Enter wizard mode password.\n\r");
      printf("  -Rfile     Restore character in master file.\n\r");
      printf("  -Ufile     Change the dead flag in save file.\n\r");
      printf("  -Efile     Encrpyt a character file.\n\r");
      printf("  -Dfile     Decrypt a character file.\n\r");
      printf("\n\r");
    } /* end if wizard */

}; /* end intro_qualifier_help */

//////////////////////////////////////////////////////////////////////
static boolean intro_parse_switches(vtype finame, int argc, char *argv[])
{
  boolean  exit_flag    = false;
  boolean  print_usage  = false;
  boolean  show_highs   = false;
  boolean  show_version = false;

  /* so far there is no filename to open */
  finame[0] = 0;
  max_score = 20;

  FILE *fp;
  static char chrsav[] = "BOSSCHR.SAV";
  int autoload = 0;

  if ((fp = fopen(chrsav, "r")) != NULL) {
    autoload = 1;
    fclose(fp);
  }

  /* parse the command line arguments */
  for (--argc, ++argv; (argc > 0) && (argv[0][0] == '-'); --argc, ++argv) {
    switch (argv[0][1])
      {
      case 'Q':
	printf("Data files already exist, exiting.\n\r");
	exit_flag = true;
	break;

      case 'V':
	/* version info */
	show_version = true;
	break;

        /* filename of character to load */
      case 'f':
        if(--argc) {
          strncpy(finame, (++argv)[0], sizeof(vtype));
        } else {
          printf("Missing <filename> for -f\n\r");
          print_usage = true;
        }
        break;

      case 'n':
        autoload = 0;
        break;

        /* print the high scores */
      case 's':
	show_highs = true;
        break;

        /* number of scores to show */
      case 't':
	if(--argc) {
	  sscanf((++argv)[0], "%ld", &max_score);
	} else {
          printf("Missing <num> for -t\n\r");
          print_usage = true;
	}
        break;

        /* kick into wizard mode */
      case 'W':
/*
	if (argv[0][2] == 0) {
	  check_pswd("",false);
	} else {
	  check_pswd(&(argv[0][2]),true);
	}
        break;
*/
        /* restore an invalid character (fix master file entry) */
/*
      case 'R':
	if (!wizard) {
	  check_pswd("",false);
	}
	if (wizard) {
	  if (argv[0][2] == 0) {
	    restore_char("", false, false);
	  } else {
	    restore_char(&(argv[0][2]), true, false);
	  }
	}
        break;
*/
        /* bring a character back from the dead (and fix master file) */
/*
      case 'U':
	if (!wizard) {
	  check_pswd("",false);
	}
	if (wizard) {
	  if (argv[0][2] == 0) {
	    restore_char("", false, true);
	  } else {
	    restore_char(&(argv[0][2]), true, true);
	  }
	}
        break;
*/
	/* encrypt a save file */
/*
      case 'E':
	if (!wizard) {
	  check_pswd("",false);
	}
	if (wizard) {
	  encrypt_file(&(argv[0][2]));
	  printf("\n\r\n\r");
	  exit_game();
	}
        break;
*/
	/* decrypt a save file */
      case 'D':
/*
	if (!wizard) {
	  check_pswd("",false);
	}
	if (wizard) {
	  decrypt_file(&(argv[0][2]));
	  printf("\n\r\n\r");
	  exit_game();
	}
        break;
*/
      default:
        printf("Unknown switch \"%s\"\n\r", argv[0]);
        print_usage = true;
        break;
      }
  }

  if (autoload == 1) {
    strncpy(finame, chrsav, strlen(chrsav)+1);
  }

  if (argc > 0) {
    strncpy(finame, argv[0], sizeof(vtype));
  }

  if (show_version) {
    print_version_info();  /* not the same as game_version in wizard.c */
    exit_game();
  }

  if (show_highs) {
    top_twenty(-1, max_score);
    exit_game();
  }
  
  if (print_usage) {
    intro_qualifier_help();
    exit_game();
  }
  
  return (exit_flag || print_usage);
  
}; /* end intro_parse_switches */
//////////////////////////////////////////////////////////////////////

void intro(vtype finam, int argc, char *argv[])
{
  //{ Attempt to open the intro file }

  vtype    in_line;
  FILE     *file1;
  GDBM_FILE file2;
  boolean  exit_flag = false;

  exit_flag = intro_do_hours_file(exit_flag,      BOSS_HOU);
  exit_flag = intro_do_msg_file(exit_flag,        BOSS_MOR, false);
//exit_flag = intro_do_master_file(exit_flag,       BOSS_MAS);
  exit_flag = intro_ensure_file_exists(exit_flag, BOSS_TOP);
//  exit_flag = intro_ensure_file_exists(exit_flag, BOSS_HLP);
//  exit_flag = intro_ensure_file_exists(exit_flag, BOSS_MON);
//  exit_flag = intro_ensure_file_exists(exit_flag, BOSS_OBJ);
//  exit_flag = intro_ensure_file_exists(exit_flag, BOSS_INV);
  exit_flag = intro_ensure_file_exists(exit_flag, BOSS_LOS);
  exit_flag = intro_ensure_file_exists(exit_flag, BOSS_ANNOY);
  exit_flag = intro_ensure_file_exists(exit_flag, BOSS_WIZ);
  exit_flag = intro_do_bus_file(exit_flag,        BUS_PIC);
  exit_flag = intro_do_quote_file(exit_flag,      QUOTES);
//  exit_flag = intro_do_death_file(exit_flag,      BOSS_DTH);
//  exit_flag = intro_ensure_file_exists(exit_flag, BOSS_GCST);

  if (exit_flag) {
    master_file_open(&file2);
    master_file_close(&file2);

    writeln("Notice: System BOSS wizard should set the protection"   );
    writeln("        on files just created. See README for"          );
    writeln("        help on setting protection on the files."       );
    writeln("        Hint: make privs"                               );
    writeln("Notice: File HOURS.DAT may be edited to set operating"  );
    writeln("        hours for BOSS."                                );
    writeln("Notice: File MESSAGE.DAT may be edited to contain news" );
    writeln("        items, etc..."                                  );

    exit_game();
  } /* end if exit_flag */

  // { Check the terminal type and see if it is supported}
  init_curses();
  termdef();
  curses_is_running = true;


  if (!exit_flag) {
    exit_flag = intro_parse_switches(finam, argc, argv);
  }
  if (exit_flag) {
    exit_game();
  } else {
    if (!wizard) {
      no_controly();
      if (already_playing()) {
        writeln ("Hey bub, you're already playing a game.");
        exit_game();
      }
    } /* end if !wizard */

    if (!check_time() && !wizard) {
      /* print out the hours file and exit the game */

      file1 = priv_fopen(BOSS_HOU,"r");
      if (file1 == NULL) {
        printf("Unable to open %s for reading\n\r", BOSS_HOU);
      } else {
        while (!feof(file1)) {
          in_line[0] = 0;
          fgets(in_line, sizeof(in_line), file1);
          printf("%s\r",in_line);
        }
        fclose(file1);
      }
      exit_game();
    } /* end if !check_time and !wizard */

    intro_do_msg_file(false, BOSS_MOR, true);

  } /* end if !exit_flag */


};

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
boolean read_top_scores(FILE **f1, char *fnam, string list[],
			int max_high, int *n1, char *openerr)
{
  /* list[] should be declared string list[max_high+1] */
  /* be sure to unlock the file when finished          */

  boolean file_flag = false;
  boolean paniced;
  ntype   temp;
  int     i1, trys;

  encrypt_state  hs_state;

  for (i1 = 1; i1 <= max_high; i1++) {
    list[i1][0] = 0;
  }

  openerr[0] = 0;
  *n1        = 0;

  *f1 = priv_fopen(fnam,"r+");
  encrypt_init(&hs_state, highScoreKey, scoresAreEncrypted);
  set_seed(ENCRYPT_SEED1);

  if (*f1 == NULL) {
    sprintf(openerr, "Error opening> %s",fnam);
  } else {

    /* get exclusive access to the high score file */

    for (trys = 0; (trys < 5) && !file_flag ; trys++) {
      if (flock((int)fileno(*f1), LOCK_EX | LOCK_NB) == 0) { 
	file_flag = true;
      } else {
	sleep(1);
      }
    }

    if ( !file_flag ) {
      fclose(*f1); *f1 = NULL;
      strcpy(openerr, "Error gaining lock for score file.");
    } else {
      for ( paniced = false, *n1 = 1; !paniced && (*n1 <= max_high) ; ) {
	read_decrypt(*f1, &hs_state, temp, &paniced);
	if (!paniced) {
	  if (strlen(temp) > 80) { temp[80] = 0; }
	  strcpy(list[*n1], temp);
	  (*n1)++;
	}
      }
    }
  }

  (*n1)--;
  return file_flag;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
boolean write_top_scores(FILE **f1, string list[], int max_high)
{
  int             i1;
  vtype           temp;
  encrypt_state   hs_state;

  rewind(*f1); ftruncate((int)fileno(*f1), 0);
  encrypt_init(&hs_state, highScoreKey, scoresAreEncrypted);
  set_seed(ENCRYPT_SEED1);
	
  for (i1 = 1; i1 <= max_high; i1++) {
    strcpy(temp, list[i1]);
    encrypt_write(*f1, &hs_state, temp);
  }

  encrypt_flush(*f1, &hs_state);
  return true;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
boolean close_top_scores(FILE **f1)
{
  boolean  return_value = true; 

  if (flock((int)fileno(*f1), LOCK_UN) != 0) {
    return_value = false;
  }
  fclose(*f1);

  return return_value;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
char *center(char *in_str, integer str_len, char *out_str)
{
  int  i, j;

  i = strlen(in_str);
  j = (str_len - i) / 2;

  if (i >= str_len) {
    strncpy(out_str, in_str, str_len);
  } else {
    sprintf(out_str, "%*s%s%*s", j, "", in_str, (int)str_len - i - j, "");
  }

  return out_str;
};
//////////////////////////////////////////////////////////////////////
char * format_top_score(vtype out_str, char *username, integer score,
                        int diffic, char *charname, int level, char *race,
                        char *class)
{
  vtype   s4;

  strcpy(s4, username);
  if (strlen(s4) > 12) { s4[12] = 0; }
      
  sprintf(out_str, "%-13s%8ld %-24s  %-2d  %-12s %s",
	  s4,score,charname,level,race,class);

  return out_str;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void character_print()
{
  /*{ Print the character to a file or device }*/
  
  integer    i1,i2;
  FILE      *file1;
  char      *prt1;
  vtype      filename1,prt2,out_val;
  stat_type  out_str,out_int,out_wis,out_dex,out_con,out_chr;
 
  prt("File name: ",1,1);
  if (get_string(filename1,1,12,64)) {
    if (filename1[0] == 0) {
      strcpy(filename1, "BOSSCHR.DAT");
    }
    file1 = (FILE *)fopen(filename1,"w");
    if (file1 != NULL) {
      prt("Writing character sheet...",1,1);
      put_qio();

      fprintf(file1, " \n \n \n");

      cnv_stat(py.stat.cstr,out_str);
      cnv_stat(py.stat.cint,out_int);
      cnv_stat(py.stat.cwis,out_wis);
      cnv_stat(py.stat.cdex,out_dex);
      cnv_stat(py.stat.ccon,out_con);
      cnv_stat(py.stat.cchr,out_chr);

      fprintf(file1, " \n \n \n");

      fprintf(file1, "  Name  : %-24s",py.misc.name);
      fprintf(file1, "  Age         :%4d",py.misc.age);
      fprintf(file1, "     Strength     : %6s\n",out_str);

      fprintf(file1, "  Qual  : %-24s",py.misc.race);
      fprintf(file1, "  Height      :%4d",py.misc.ht);
      fprintf(file1, "     Intelligence : %6s\n",out_int);

      fprintf(file1, "  Sex   : %-24s",py.misc.sex);
      fprintf(file1, "  Weight      :%4d",py.misc.wt);
      fprintf(file1, "     Wisdom       : %6s\n",out_wis);

      fprintf(file1, "  Class : %-24s",py.misc.tclass);
      fprintf(file1, "  Social Class:%4d",py.misc.sc);
      fprintf(file1, "     Dexterity    : %6s\n",out_dex);

      fprintf(file1, "  Title : %-24s",py.misc.title);
      fprintf(file1, "                   ");
      fprintf(file1, "     Constitution : %6s\n",out_con);

      fprintf(file1, "          %24s","");
      fprintf(file1, "                   ");
      fprintf(file1, "     Charisma     : %6s\n",out_chr);

      fprintf(file1, " \n \n \n \n");

      fprintf(file1, "  + To Hit    :%6d",py.misc.dis_th);
      fprintf(file1, "     Level      :%6d",py.misc.lev);
      fprintf(file1, "     Max Hit Points :%6d\n",py.misc.mhp);

      fprintf(file1, "  + To Damage :%6d",py.misc.dis_td);
      fprintf(file1, "     Experience :%6ld",py.misc.exp);
      fprintf(file1, "     Cur Hit Points :%6d\n",(int)(py.misc.chp));

      fprintf(file1, "  + To AC     :%6d",py.misc.dis_tac);
      fprintf(file1, "     Cash       :%6ld",py.misc.au);
      fprintf(file1, "     Max Resources  :%6d\n",py.misc.mana);

      fprintf(file1, "    Total AC  :%6d",py.misc.dis_ac);
      fprintf(file1, "                       ");
      fprintf(file1, "     Current Rsrc   :%6d\n",py.misc.mana);
 
      fprintf(file1, " \n \n");

      fprintf(file1, "(Miscellaneous Abilities)");
      fprintf(file1, " \n");

      fprintf(file1, "  Melee Fights: %10s",likert(xbth() ,20,1));
      fprintf(file1, "  Stealth     : %10s",likert(xstl() ,10,1));
      fprintf(file1, "  Perception  : %10s\n",likert(xperc(),10,1));

      fprintf(file1, "  Guns/Grenade: %10s",likert(xbthb(),20,1));
      fprintf(file1, "  Disarming   : %10s",likert(xdis() ,10,1));
      fprintf(file1, "  Searching   : %10s\n",likert(xsrh() ,10,1));

      fprintf(file1, "  Saving Throw: %10s",likert(xsave(),10,1));
      fprintf(file1, "  Weird Device: %10s",likert(xdev() ,10,1));
      fprintf(file1, "  Infra-Vision: %10s\n",xinfra(out_val));

      /*{ Write out the character's history }*/
      fprintf(file1, " \n \n");
      fprintf(file1, "Character Background\n");
      for (i1 =0; i1 < 5; i1++) {
	fprintf(file1, "%s\n",py.misc.history[i1]);
      }

      /*{ Write out the equipment list.}*/
      i2 = 0;
      fprintf(file1, " \n \n");
      fprintf(file1, "  [Character's Equipment List]\n");
      fprintf(file1, " \n");

      if (equip_ctr == 0) {
	fprintf(file1, "  Character has no equipment in use.\n");
      } else {
	for (i1 = 23; i1 <= EQUIP_MAX; i1++) {
	  //with equipment[i1]. do;
	  if (equipment[i1].tval > 0) {
	    switch (i1) {
	    case 23 :        prt1 = ") You are wielding   : ";
	    case 24 :        prt1 = ") Worn on head       : ";
	    case 25 :        prt1 = ") Worn around neck   : ";
	    case 26 :        prt1 = ") Worn on body       : ";
	    case 27 :        prt1 = ") Worn on shield arm : ";
	    case 28 :        prt1 = ") Worn on hands      : ";
	    case 29 :        prt1 = ") Right ring finger  : ";
	    case 30 :        prt1 = ") Left  ring finger  : ";
	    case 31 :        prt1 = ") Worn on feet       : ";
	    case 32 :        prt1 = ") Worn about body    : ";
	    case 33 :        prt1 = ") Light source is    : ";
	    case 34 :        prt1 = ") Secondary weapon   : ";
	    case 35 :        prt1 = ") Computer Used      : ";
	    default :        prt1 = ") *Unknown value*    : ";
	    }
	    i2++;
	    objdes(prt2,i1,true,'e');
	    fprintf(file1, "  %c%s%s\n",(int)i2+96,prt1,prt2);
	  }
	}
      }
      
      /*{ Write out the character's inventory.}*/
      //writeln(file1,new_page,);
      fprintf(file1, " \n \n \n");
      fprintf(file1, "  [General Inventory List]\n");
      fprintf(file1, " \n");

      if (inven_ctr == 0) {
	fprintf(file1,"  Character has no objects in inventory.\n");
      } else {
	for (i1 = 1; i1 <= inven_ctr; i1++) {
	  objdes(prt2,i1,true,'i');
	  fprintf(file1,"%c) %s\n",(int)i1+96, prt2);
	}
      }

      //writeln(file1,new_page);

      fclose(file1);
      prt("Completed.",1,1);
    } /* end if file !NULL */
  } /* end get_string */ 
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void read_text_data(integer position, char *filename)
{
  integer   num,starts,msg_lines;
  FILE     *infile;
  vtype     string;

  infile = priv_fopen( filename, "r");
  if (infile == NULL) {
    sprintf(string, "Unable to open %s for reading", filename);
    msg_print(string);
  } else {
    fgets(string, sizeof(string), infile);
    sscanf(string, "%ld", &num);
    if (position > num) {
      sprintf(string, "Error: Message number %ld in file %s",
	      position, filename);
      msg_print(string);
      msg_print("Please report error to your local BOSS wizard.");
    } else {
      if (position == 0) {
        position = randint(num);
      }

      starts = 1;
      while(starts < position) {
	fgets(string, sizeof(string), infile);
	if (!strcmp(string, "**")) {
	  starts++;
	}
      }
      
      msg_lines = 0;
      fgets(string, sizeof(string), infile);

      while (!strcmp(string,"**")) {
        msg_lines++;
        msg_print(string);
	fgets(string, sizeof(string), infile);
      }
      
      if (msg_lines == 0) {
	sprintf(string,"Error: blank message number %ld in file %s",
		position, filename);
	msg_print(string);
	msg_print("Please report error to your local BOSS wizard.");
      }
    }
    fclose(infile);
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void print_map()
{
  /*{ Prints dungeon map to external file }*/

  integer    i1,i2,i3,i4,i5,i6,i7,i8;
  char       dun_line[134];
  char       filename1[81];
  char       tmp;
  FILE      *file1;

  prt("File name: ",1,1);
  if (get_string(filename1,1,12,64)) {
    if (strlen(filename1) == 0) {
      strcpy(filename1, "BOSSMAP.DAT");
    }
    file1 = (FILE *)fopen(filename1,"w");
    if (file1 != NULL) {
      prt("Writing Boss Dungeon Map...",1,1);
      put_qio();
      i1 = 1;
      i7 = 0;
      do {
        i2 = 1;
        i3 = i1 + OUTPAGE_HEIGHT - 1;
        if (i3 > cur_height) {
          i3 = cur_height;
        }
        i7++;
        i8 = 0;
        do {
          i4 = i2 + OUTPAGE_WIDTH - 1;
          if (i4 > cur_width) {
            i4 = cur_width;
          }
          i8++;
          fprintf(file1,"%c",12);
          fprintf(file1,"Section[%ld,%ld];     ",i7,i8);
          fprintf(file1,"Depth : %ld (feet)\n",(dun_level*50));
          fprintf(file1," \n");
          fprintf(file1,"   ");
          for (i5 = i2; i5 <= i4; i5++) {
            i6 = i5/100;
            fprintf(file1,"%ld",i6);
          }
          fprintf(file1,"\n");
          fprintf(file1,"   ");
          for (i5 = i2; i5 <= i4; i5++) {
            i6 = (int)(i5/10) - (int)(i5/100)*10;
            fprintf(file1,"%ld",i6);
          }
          fprintf(file1,"\n");
          fprintf(file1,"   ");
          for (i5 = i2; i5 <= i4; i5++) {
            i6 = i5 - (int)(i5/10)*10;
            fprintf(file1,"%ld",i6);
          }
          fprintf(file1,"\n");
          for (i5 = i1; i5 < i3; i5++) {
            sprintf(dun_line,"%3ld",i5);
            for (i6 = i2; i6 <= i4; i6++) {
              if (test_light(i5,i6)) {
                tmp = loc_symbol(i5,i6);
              } else {
                tmp = ' ';
              }
              strncat(dun_line, &tmp, 1);
            }
            fprintf(file1,"%s\n",dun_line);
          }
          i2 += OUTPAGE_WIDTH;
        } while (i2 < cur_width);
        i1 += OUTPAGE_HEIGHT;
      } while (i1 < cur_height);
      fclose(file1);
      prt("Completed.",1,1);
    }   
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

/* END FILE files.c */
