/* generate.c */
/**/

#include "boss.h"

typedef struct coords
{
  integer   y;
  integer   x;
} coords;

typedef struct spot_type
{
  integer ENDx;
  integer ENDy;
} spot_type;

static coords   doorstk[100+1]; // : array [1..100] of coords;
static integer  doorptr;

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
static void correct_dir(integer *rdir, integer *cdir, 
			integer y1, integer x1, integer y2, integer x2)
{
    /*{ Always picks a correct direction }*/

  if (y1 < y2) {
    *rdir =  1;
  } else if (y1 == y2) {
    *rdir =  0;
  } else {
    *rdir = -1;
  }

  if (x1 < x2) {
    *cdir =  1;
  } else if (x1 == x2) {
    *cdir =  0;
  } else {
    *cdir = -1;
  }

  if ((*rdir != 0) && (*cdir != 0)) {
    switch (randint(2)) {
    case 1 :     *rdir = 0;   break;
    case 2 :     *cdir = 0;   break;
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void rand_dir(integer *rdir, integer *cdir,
		     integer y1, integer x1, integer y2, integer x2,
		     integer chance)
{
  /*{ Chance of wandering direction }*/
  switch (randint(chance)) {
  case 1 :     
    *rdir = -1;
    *cdir =  0;
    break;
  case 2 :     
    *rdir =  1;
    *cdir =  0;
    break;
  case 3 :     
    *rdir =  0;
    *cdir = -1;
    break;
  case 4 :     
    *rdir =  0;
    *cdir =  1;
    break;
  default:
    correct_dir(rdir,cdir,y1,x1,y2,x2);
    break;
  }
};
//////////////////////////////////////////////////////////////////////
static void fill_cave(floor_type fill)
{
  /*
   * { Fills in empty spots with desired rock
   *   Note: 9 is a temporary value. }
   */

  integer   i1,i2;

  obj_set   zero_eight_nine  = {0,8,9};

  for (i1 = 2; i1 <= cur_height-1; i1++) {
    for (i2 = 2; i2 <= cur_width-1; i2++) {
      //with cave[i1][i2]. do;
      if (is_in(cave[i1][i2].fval,zero_eight_nine)) {
	cave[i1][i2].fval  = fill.ftval;
	cave[i1][i2].fopen = fill.ftopen;
      }
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void place_streamer(floor_type rock, integer treas_chance)
{
  /*{ Places "streamers" of rock through dungeon }*/

  integer  i1,y,x,dir,ty,tx,t1,t2;
  boolean  flag;
 
  /*{ Choose starting point and direction }*/
  y = trunc(cur_height/2.0) + 11 - randint(23);
  x = trunc(cur_width/2.0)  + 16 - randint(33);
 
  dir = randint(8);      /*{ Number 1-4, 6-9 }*/
  if (dir > 4) {
    dir++;
  } 

/*{ Place streamer into dungeon }*/
  flag = false;  /*{ Set to true when (y,x) is out-of-bounds}*/
  t1   = 2*DUN_STR_RNG + 1;        /*{ Constants     }*/
  t2   =   DUN_STR_RNG + 1;

  do {
    for (i1 = 1; i1 <= DUN_STR_DEN; i1++) {
      ty = y + randint(t1) - t2;
      tx = x + randint(t1) - t2;
      if (in_bounds(ty,tx)) {
	//with cave[ty][tx]. do;
	if (cave[ty][tx].fval == rock_wall1.ftval) {
	  cave[ty][tx].fval  = rock.ftval;
	  cave[ty][tx].fopen = rock.ftopen;
	  if (randint(treas_chance) == 1) {
	    place_gold(ty,tx);
	  }
	}
      }
      if (!(move_dir(dir,&y,&x))) {
	flag = true;
      }
    }
  } while (!flag);
};
//////////////////////////////////////////////////////////////////////
static void vault_trap(integer y,integer x,integer yd,integer xd,integer num)
{
  /*{ Place a trap with a given displacement of point }*/

  integer   count,y1,x1,i1;
  boolean   flag;

  for (i1 = 1; i1 <= num; i1++) {
    flag  = false;
    count = 0;
    do {
      y1 = y - yd - 1 + randint(2*yd+1);
      x1 = x - xd - 1 + randint(2*xd+1);
      //with cave[y1][x1]. do;
      if (is_in(cave[y1][x1].fval, floor_set)) {
	if (cave[y1][x1].tptr == 0) {
	  place_trap(y1,x1,1,randint(MAX_TRAPA));
	  flag = true;
	}
      }
      count++;
    } while (!((flag) || (count > 5)));
  }
};
//////////////////////////////////////////////////////////////////////
static void vault_monster(integer y,integer x,integer num)
{
  /*{ Place a monster next to the given point }*/

  integer   i1,y1,x1;

  for (i1 = 1; i1 <= num; i1++) {
    y1 = y;
    x1 = x;
    summon_monster(&y1,&x1,true);
  }
};
//////////////////////////////////////////////////////////////////////
static void build_room(integer yval,integer xval)
{
  /*{ Builds a room at a row,column coordinate }*/

  integer     y_height,y_depth,x_left,x_right,i1,i2;
  floor_type  cur_floor;

  if (dun_level <= randint(10)) {
    cur_floor = lopen_floor;      /*{ Floor with light }*/
  } else {
    cur_floor = dopen_floor;     /*{ Dark floor }*/
  }

  y_height = yval - randint(4);
  y_depth  = yval + randint(3);
  x_left   = xval - randint(11);
  x_right  = xval + randint(11);

  for (i1 = y_height; i1 <= y_depth; i1++) {
    for (i2 = x_left; i2 <= x_right; i2++) {
      cave[i1][i2].fval  = cur_floor.ftval;
      cave[i1][i2].fopen = cur_floor.ftopen;
    }
  }

  for (i1 = (y_height - 1); i1 <= (y_depth + 1); i1++) {
    cave[i1][x_left-1].fval   = rock_wall1.ftval;
    cave[i1][x_left-1].fopen  = rock_wall1.ftopen;
    cave[i1][x_right+1].fval  = rock_wall1.ftval;
    cave[i1][x_right+1].fopen = rock_wall1.ftopen;
  }
  
  for (i1 = x_left; i1 <= x_right; i1++) {
    cave[y_height-1][i1].fval  = rock_wall1.ftval;
    cave[y_height-1][i1].fopen = rock_wall1.ftopen;
    cave[y_depth+1][i1].fval   = rock_wall1.ftval;
    cave[y_depth+1][i1].fopen  = rock_wall1.ftopen;
  }
};
//////////////////////////////////////////////////////////////////////
static void build_type1(integer yval, integer xval)
{
  /*{ Builds a room at a row,column coordinate
    Type 1 unusual rooms are several overlapping rectangular ones }*/

  integer     y_height,y_depth,x_left,x_right,i0,i1,i2;
  floor_type  cur_floor;


  if (dun_level <= randint(10)) {
    cur_floor = lopen_floor;      /*{ Floor with light }*/
  } else {
    cur_floor = dopen_floor;     /*{ Dark floor }*/
  }

  for (i0 = 1; i0 <= (1 + randint(2)) ; i0++) {

    y_height = yval - randint(4);
    y_depth  = yval + randint(3);
    x_left   = xval - randint(11);
    x_right  = xval + randint(11);

    for (i1 = y_height; i1 <= y_depth ; i1++) {
      for (i2 = x_left; i2 <= x_right ; i2++) {
	cave[i1][i2].fval  = cur_floor.ftval;
	cave[i1][i2].fopen = cur_floor.ftopen;
      }
    }
    
    for (i1 = (y_height - 1); i1 <= (y_depth + 1); i1++) {
      //with cave[i1][x_left-1]. do;
      if (cave[i1][x_left-1].fval != cur_floor.ftval) {
	cave[i1][x_left-1].fval  = rock_wall1.ftval;
	cave[i1][x_left-1].fopen = rock_wall1.ftopen;
      }

      //with cave[i1][x_right+1]. do;
      if (cave[i1][x_right+1].fval != cur_floor.ftval) {
	cave[i1][x_right+1].fval  = rock_wall1.ftval;
	cave[i1][x_right+1].fopen = rock_wall1.ftopen;
      }
    }
    
    for (i1 = x_left; i1 <= x_right; i1++) {
      
      //with cave[y_height-1][i1]. do;
      if (cave[y_height-1][i1].fval != cur_floor.ftval) {
	cave[y_height-1][i1].fval  = rock_wall1.ftval;
	cave[y_height-1][i1].fopen = rock_wall1.ftopen;
      }
      
      //with cave[y_depth+1][i1]. do;
      if (cave[y_depth+1][i1].fval != cur_floor.ftval) {
	cave[y_depth+1][i1].fval  = rock_wall1.ftval;
	cave[y_depth+1][i1].fopen = rock_wall1.ftopen;
      }
    }

  }
};
//////////////////////////////////////////////////////////////////////
static void build_type2(integer yval, integer xval)
{
/*{ Builds an unusual room at a row,column coordinate
  Type 2 unusual rooms all have an inner room:
   1 - Just an inner room with one door
   2 - An inner room within an inner room
   3 - An inner room with pillar(s)
   4 - Inner room has a maze
   5 - A set of four inner rooms }*/

  integer     y_height,y_depth,x_left,x_right,i1,i2;
  floor_type  cur_floor;

  if (dun_level <= randint(10)) {
    cur_floor = lopen_floor;      /*{ Floor with light }*/
  } else {
    cur_floor = dopen_floor;     /*{ Dark floor }*/
  }

  y_height = yval - 4;
  y_depth  = yval + 4;
  x_left   = xval - 11;
  x_right  = xval + 11;
  
  for (i1 = y_height; i1 <= y_depth; i1++) {
    for (i2 = x_left; i2 <= x_right; i2++) {
      cave[i1][i2].fval  = cur_floor.ftval;
      cave[i1][i2].fopen = cur_floor.ftopen;
    }
  }
  
  for (i1 = (y_height - 1); i1 <= (y_depth + 1); i1++) {
    cave[i1][x_left-1].fval   = rock_wall1.ftval;
    cave[i1][x_left-1].fopen  = rock_wall1.ftopen;
    cave[i1][x_right+1].fval  = rock_wall1.ftval;
    cave[i1][x_right+1].fopen = rock_wall1.ftopen;
  }
  
  for (i1 = x_left; i1 <= x_right; i1++) {
    cave[y_height-1][i1].fval  = rock_wall1.ftval;
    cave[y_height-1][i1].fopen = rock_wall1.ftopen;
    cave[y_depth+1][i1].fval   = rock_wall1.ftval;
    cave[y_depth+1][i1].fopen  = rock_wall1.ftopen;
  }
  
  /*{ The inner room }*/
  
  y_height = y_height + 2;
  y_depth  = y_depth  - 2;
  x_left   = x_left   + 2;
  x_right  = x_right  - 2;
  
  for (i1 = (y_height - 1); i1 <= (y_depth + 1); i1++) {
    cave[i1][x_left-1].fval   = 8;
    cave[i1][x_right+1].fval  = 8;
  }
  
  for (i1 = x_left; i1 <= x_right; i1++) {
    cave[y_height-1][i1].fval  = 8;
    cave[y_depth+1][i1].fval   = 8;
  }
  
  /*{ Inner room varitions }*/
  
  switch (randint(5)) {
  case 1 :      /*{ Just an inner room... }*/
    switch (randint(4)) {     /*{ Place a door  }*/
    case 1 : place_secret_door(y_height-1,xval); break;
    case 2 : place_secret_door(y_depth+1,xval); break;
    case 3 : place_secret_door(yval,x_left-1); break;
    case 4 : place_secret_door(yval,x_right+1); break;
    }
    vault_monster(yval,xval,1);
    break;
    
  case 2 :      /*{ Treasure Vault }*/
    switch (randint(4)) {    /*{ Place a door  }*/
    case 1 : place_secret_door(y_height-1,xval); break;
    case 2 : place_secret_door(y_depth+1,xval);  break;
    case 3 : place_secret_door(yval,x_left-1);   break;
    case 4 : place_secret_door(yval,x_right+1);  break;
    }
    
    for (i1 = yval-1; i1 <= yval+1; i1++) {
      cave[i1][xval-1].fval   = 8;
      cave[i1][xval+1].fval   = 8;
    }
    
    cave[yval-1][xval].fval  = 8;
    cave[yval+1][xval].fval  = 8;
    
    switch (randint(4)) {    /*{ Place a door  }*/
    case 1 : place_locked_door(yval-1,xval); break;
    case 2 : place_locked_door(yval+1,xval); break;
    case 3 : place_locked_door(yval,xval-1); break;
    case 4 : place_locked_door(yval,xval+1); break;
    }
    
    /*{ Place an object in the treasure vault }*/
    switch (randint(10)) {
    case 1 : place_up_stairs(yval,xval);    break;
    case 2 : place_down_stairs(yval,xval);  break;
    default: place_object(yval,xval);       break;
    }
    
    /*{ Guard the treasure well }*/
    vault_monster(yval,xval,2+randint(3));
    
    /*{ If the monsters don't get 'em.}*/
    vault_trap(yval,xval,4,10,2+randint(3));
    break;
	    
  case 3 :      /*{ Inner pillar(s).}*/
    switch (randint(4)) {    /*{ Place a door  }*/
    case 1 : place_secret_door(y_height-1,xval); break;
    case 2 : place_secret_door(y_depth+1,xval);  break;
    case 3 : place_secret_door(yval,x_left-1);   break;
    case 4 : place_secret_door(yval,x_right+1);  break;
    }

    for (i1 = yval-1; i1 <= yval+1; i1++) {
      for (i2 = xval-1; i2 <= xval+1; i2++) {
	cave[i1][i2].fval   = 8;
      }
    }

    if (randint(2) == 1) {
      switch (randint(2)) {
      case 1 : 
	for (i1 = yval-1; i1 <= yval+1; i1++) {
	  for (i2 = xval-6; i2 <= xval-4; i2++) {
	    cave[i1][i2].fval   = 8;
	  }
	}
	for (i1 = yval-1; i1 <= yval+1; i1++) {
	  for (i2 = xval+4; i2 <= xval+6; i2++) {
	    cave[i1][i2].fval   = 8;
	  }
	}
	break;
	
	
      case 2 : 
	for (i1 = yval-1; i1 <= yval+1; i1++) {
	  for (i2 = xval-7; i2 <= xval-5; i2++) {
	    cave[i1][i2].fval   = 8;
	  }
	}
	for (i1 = yval-1; i1 <= yval+1; i1++) {
	  for (i2 = xval+5; i2 <= xval+7; i2++) {
	    cave[i1][i2].fval   = 8;
	  }
	}
	break;
      }
      
      
      if (randint(3) == 1) {  /*{ Inner rooms   }*/
	for (i1 = xval-5; i1 <= xval+5; i1++) {
	  cave[yval-1][i1].fval = 8;
	  cave[yval+1][i1].fval = 8;
	}
	
	switch (randint(2)) {
	case 1 : place_secret_door(yval+1,xval-3); break;
	case 2 : place_secret_door(yval-1,xval-3); break;
	}
	
	switch (randint(2)) {
	case 1 : place_secret_door(yval+1,xval+3); break;
	case 2 : place_secret_door(yval-1,xval+3); break;
	}
	
	if (randint(3) == 1) { place_object(yval,xval-2); }
	if (randint(3) == 1) { place_object(yval,xval+2); }
	vault_monster(yval,xval-2,randint(2));
	vault_monster(yval,xval+2,randint(2));
      }
    }
    break;
    
    
  case 4 :      /*{ Maze inside..}*/
    switch (randint(4)) {    /*{ Place a door  }*/
    case 1 : place_secret_door(y_height-1,xval); break;
    case 2 : place_secret_door(y_depth+1,xval);  break;
    case 3 : place_secret_door(yval,x_left-1);   break;
    case 4 : place_secret_door(yval,x_right+1);  break;
    }
    
    for (i1 = y_height; i1 <= y_depth; i1++) {
      for (i2 = x_left; i2 <= x_right; i2++) {
	if ((i2+i1) & 1) {
	  cave[i1][i2].fval = 8;
	}
      }
    }
    
    /*{ Monsters just love mazes.}*/
    vault_monster(yval,xval-5,randint(3));
    vault_monster(yval,xval+5,randint(3));
    
    /*{ Traps make them entertaining.}*/
    vault_trap(yval,xval-3,2,8,randint(3));
    vault_trap(yval,xval+3,2,8,randint(3));
    
    /*{ Mazes should have some treasure too.. }*/
    for (i1 = 1; i1 <= 6; i1++) {
      random_object(yval,xval,1);
    }
    break;
    
  case 5 :  /*{ four small rooms ...}*/
    for (i1 = y_height; i1 <= y_depth; i1++) {
      cave[i1][xval].fval = 8;
    }
    for (i1 = x_left; i1 <= x_right; i1++) {
      cave[yval][i1].fval = 8;
    }
    switch (randint(3)) {
    case 1 : 
      i1 = randint(10);
      place_secret_door(y_height-1,xval-i1);
      place_secret_door(y_height-1,xval+i1);
      place_secret_door(y_depth+1,xval-i1);
      place_secret_door(y_depth+1,xval+i1);
      break;
      
    case 2 : 
      i1 = randint(3);
      place_secret_door(yval+i1,x_left-1);
      place_secret_door(yval-i1,x_left-1);
      place_secret_door(yval+i1,x_right+1);
      place_secret_door(yval-i1,x_right+1);
      break;
      
    case 3 :  /*{ make them dig for it ... REB}*/
      place_secret_door(yval,xval);
      place_secret_door(y_height,xval);
      break;
    }
    break;
    
  }
  
  /*{ Treasure in each one.}*/
  random_object(yval,xval,2+randint(2));

  /*{ Gotta have some monsters.}*/
  vault_monster(yval+2,xval-4,randint(2));
  vault_monster(yval+2,xval+4,randint(2));
  vault_monster(yval-2,xval-4,randint(2));
  vault_monster(yval-2,xval+4,randint(2));
};
//////////////////////////////////////////////////////////////////////
static void build_type3(integer yval, integer xval)
{
  /*{ Builds a room at a row,column coordinate
  Type 3 unusual rooms are cross shaped }*/

  integer     y_height,y_depth,x_left,x_right,i0,i1,i2;
  floor_type  cur_floor;

  if (dun_level <= randint(10)) {
    cur_floor = lopen_floor;      /*{ Floor with light }*/
  } else {
    cur_floor = dopen_floor;     /*{ Dark floor }*/
  }
  
  i0 = 2 + randint(2);
  y_height = yval - i0;
  y_depth  = yval + i0;
  x_left   = xval - 1;
  x_right  = xval + 1;
  
  for (i1 = y_height; i1 <= y_depth; i1++) {
    for (i2 = x_left; i2 <= x_right; i2++) {
      cave[i1][i2].fval  = cur_floor.ftval;
      cave[i1][i2].fopen = cur_floor.ftopen;
    }
  }


  for (i1 = (y_height - 1); i1 <= (y_depth + 1); i1++) {
    //with cave[i1][x_left-1]. do;
    cave[i1][x_left-1].fval  = rock_wall1.ftval;
    cave[i1][x_left-1].fopen = rock_wall1.ftopen;

    //with cave[i1][x_right+1]. do;
    cave[i1][x_right+1].fval  = rock_wall1.ftval;
    cave[i1][x_right+1].fopen = rock_wall1.ftopen;
  }


  for (i1 = x_left; i1 <= x_right; i1++) {
    //with cave[y_height-1][i1]. do;
    cave[y_height-1][i1].fval  = rock_wall1.ftval;
    cave[y_height-1][i1].fopen = rock_wall1.ftopen;

    //with cave[y_depth+1][i1]. do;
    cave[y_depth+1][i1].fval  = rock_wall1.ftval;
    cave[y_depth+1][i1].fopen = rock_wall1.ftopen;
  }


  i0 = 2 + randint(9);
  y_height = yval - 1;
  y_depth  = yval + 1;
  x_left   = xval - i0;
  x_right  = xval + i0;

  for (i1 = y_height; i1 <= y_depth; i1++) {
    for (i2 = x_left; i2 <= x_right; i2++) {
      cave[i1][i2].fval  = cur_floor.ftval;
      cave[i1][i2].fopen = cur_floor.ftopen;
    }
  }

  for (i1 = (y_height - 1); i1 <= (y_depth + 1); i1++) {

    //with cave[i1][x_left-1]. do;
    if (cave[i1][x_left-1].fval != cur_floor.ftval) {
      cave[i1][x_left-1].fval  = rock_wall1.ftval;
      cave[i1][x_left-1].fopen = rock_wall1.ftopen;
    }

    //with cave[i1][x_right+1]. do;
    if (cave[i1][x_right+1].fval != cur_floor.ftval) {
      cave[i1][x_right+1].fval  = rock_wall1.ftval;
      cave[i1][x_right+1].fopen = rock_wall1.ftopen;
    }
  }


  for (i1 = x_left; i1 <= x_right; i1++) {
    //with cave[y_height-1][i1]. do;
    if (cave[y_height-1][i1].fval != cur_floor.ftval) {
      cave[y_height-1][i1].fval  = rock_wall1.ftval;
      cave[y_height-1][i1].fopen = rock_wall1.ftopen;
    }

    //with cave[y_depth+1][i1]. do;
    if (cave[y_depth+1][i1].fval != cur_floor.ftval) {
      cave[y_depth+1][i1].fval  = rock_wall1.ftval;
      cave[y_depth+1][i1].fopen = rock_wall1.ftopen;
    }
  }

  /*{ Special features.}*/
  switch (randint(4)) {
  case 1 :      /*{ Large middle pillar  }*/
    for (i1 = yval-1; i1 <= yval+1; i1++) {
      for (i2 = xval-1; i2 <= xval+1; i2++) {
	cave[i1][i2].fval = 8;
      }
    }
    break;
		  
  case 2 :      /*{ Inner treasure vault }*/
    for (i1 = yval-1; i1 <= yval+1; i1++) {
      cave[i1][xval-1].fval   = 8;
      cave[i1][xval+1].fval   = 8;
    }
    cave[yval-1][xval].fval  = 8;
    cave[yval+1][xval].fval  = 8;
    switch (randint(4)) {    /*{ Place a door  }*/
    case 1 : place_secret_door(yval-1,xval); break;
    case 2 : place_secret_door(yval+1,xval); break;
    case 3 : place_secret_door(yval,xval-1); break;
    case 4 : place_secret_door(yval,xval+1); break;
    }
    /*{ Place a treasure in the vault }*/
    place_object(yval,xval);
    /*{ Let's gaurd the treasure well.}*/
    vault_monster(yval,xval,2+randint(2));
    /*{ Traps naturally }*/
    vault_trap(yval,xval,4,4,1+randint(3));
    
    break;
    
  case 3 :   
    if (randint(3) == 1) {
      
      cave[yval-1][xval-2].fval = 8;
      cave[yval+1][xval-2].fval = 8;
      cave[yval-1][xval+2].fval = 8;
      cave[yval-1][xval+2].fval = 8;
      cave[yval-2][xval-1].fval = 8;
      cave[yval-2][xval+1].fval = 8;
      cave[yval+2][xval-1].fval = 8;
      cave[yval+2][xval+1].fval = 8;
      
      if (randint(3) == 1) {
	place_secret_door(yval,xval-2);
	place_secret_door(yval,xval+2);
	place_secret_door(yval-2,xval);
	place_secret_door(yval+2,xval);
      }
    } else if (randint(3) == 1) {
      
      cave[yval  ][xval  ].fval = 8;
      cave[yval-1][xval  ].fval = 8;
      cave[yval+1][xval  ].fval = 8;
      cave[yval  ][xval-1].fval = 8;
      cave[yval  ][xval+1].fval = 8;
      
      
    } else if (randint(3) == 1) {
      cave[yval][xval].fval = 8;
    }
    break;
    
  case 4 :   ;
    break;
  }
};
//////////////////////////////////////////////////////////////////////
static void build_type4(integer yval, integer xval)
{
  /*{Builds a room with a circular inner room. - RLG}*/ 

  integer     i5,i6;
  integer     y_height,y_depth,x_left,x_right;
  floor_type  cur_floor;

  if (dun_level <= randint(10)) {
    cur_floor = lopen_floor;      /*{ Floor with light }*/
  } else {
    cur_floor = dopen_floor;     /*{ Dark floor }*/
  }
  
  y_height = yval - 4;
  y_depth  = yval + 4;
  x_left   = xval - 11;
  x_right  = xval + 11;
  
  for (i5 = y_height; i5 <= y_depth; i5++) {
    for (i6 = x_left; i6 <= x_right; i6++) {
      cave[i5][i6].fval  = cur_floor.ftval;
      cave[i5][i6].fopen = cur_floor.ftopen;
    }
  }
  
  for (i5 = 1; i5 <= 9; i5++) {
    i6 = i5-1;
    cave[yval+circle[i5]][xval+i6].fval  = 8;
    cave[yval-circle[i5]][xval+i6].fval  = 8;
    cave[yval+circle[i5]][xval-i6].fval  = 8;
    cave[yval-circle[i5]][xval-i6].fval  = 8;
  }

  vault_trap(yval,xval,2,4,2);
  random_object(yval,xval,2);
  vault_monster(yval,xval,3);
}; 
//////////////////////////////////////////////////////////////////////
static void build_type5(integer yval, integer xval)
{
  /*{ Builds collapsed room  --jeb}*/

  integer     i1, i2;
  integer     y_height,y_depth,x_left,x_right;
  floor_type  cur_floor;

  y_height = yval - 4;
  y_depth  = yval + 4;
  x_left   = xval - 11;
  x_right  = xval + 11;

  if (dun_level <= randint(10)) {
    cur_floor = lopen_floor;      /*{ Floor with light }*/
  } else {
    cur_floor = dopen_floor;     /*{ Dark floor }*/
  }

  for (i1 = y_height; i1 <= y_depth; i1++) {
    for (i2 = x_left  ; i2 <= x_right; i2++) {
      if (randint(4) > 1) {
	cave[i1][i2].fval  = cur_floor.ftval;
	cave[i1][i2].fopen = cur_floor.ftopen;
      } else {
	cave[i1][i2].fval  = 8;  /*{non-tunnelable rock}*/
	cave[i1][i2].fopen = false;
      }
    }
  }
  
  vault_monster(yval,xval,5);
  vault_trap(yval,xval,3,10,5);
  random_object(yval,xval,5);
}; 
//////////////////////////////////////////////////////////////////////
static void tunnel(integer row1,integer col1, integer row2, integer col2)
{
  /*{ Constructs a tunnel between two points }*/

  integer    tmp_row,tmp_col,row_dir,col_dir,i1,i2;
  coords     tunstk[10001]; //: array [1..1000] of coords;
  coords     wallstk[10001]; // : array [1..1000] of coords;
  integer    tunptr,wallptr;
  boolean    stop_flag,door_flag;
 
  /*{ Main procedure for Tunnel
    Note: 9 is a temporary value }*/

  stop_flag = false;
  door_flag = false;
  tunptr    = 0;
  wallptr   = 0;
  correct_dir(&row_dir,&col_dir,row1,col1,row2,col2);

  do {
    if (randint(100) > DUN_TUN_CHG) {
      rand_dir(&row_dir,&col_dir,row1,col1,row2,col2,DUN_TUN_RND);
    }
    
    tmp_row = row1 + row_dir;
    tmp_col = col1 + col_dir;
    
    while (!(in_bounds(tmp_row,tmp_col))) {
      rand_dir(&row_dir,&col_dir,row1,col1,row2,col2,DUN_TUN_RND);
      tmp_row = row1 + row_dir;
      tmp_col = col1 + col_dir;
    }
    
    //with cave[tmp_row][tmp_col]. do;
    if (cave[tmp_row][tmp_col].fval == rock_wall1.ftval) {
      
      row1 = tmp_row;
      col1 = tmp_col;
      if (wallptr < 1000) {
	wallptr = wallptr + 1;
      }

      wallstk[wallptr].y = row1;
      wallstk[wallptr].x = col1;
      
      for (i1 = row1-1; i1 <= row1+1; i1++) {
	for (i2 = col1-1; i2 <= col1+1; i2++) {
	  if (in_bounds(i1,i2)) {
	    //with cave[i1][i2]. do;
	    if (is_in(cave[i1][i2].fval, wall_set)) {
	      cave[i1][i2].fval = 9;
	    }
	  }
	}
      }
      
    } else if (cave[tmp_row][tmp_col].fval == corr_floor1.ftval) {

      row1 = tmp_row;
      col1 = tmp_col;
      if (!door_flag) {
	if (doorptr <= 100) {
	  doorptr = doorptr + 1;
	  doorstk[doorptr].y = row1;
	  doorstk[doorptr].x = col1;
	}
	door_flag = true;
      }
      
      if (randint(100) > DUN_TUN_CON) {
	stop_flag = true;
      }
      
    } else if (cave[tmp_row][tmp_col].fval == 0) {
      
      row1 = tmp_row;
      col1 = tmp_col;
      if (tunptr < 1000) {
	tunptr = tunptr + 1;
      }
      
      tunstk[tunptr].y = row1;
      tunstk[tunptr].x = col1;
      door_flag = false;
      
    } else if (cave[tmp_row][tmp_col].fval != 9) {

      row1 = tmp_row;
      col1 = tmp_col;
    }
    
  } while (! (((row1 == row2) && (col1 == col2)) || (stop_flag)));


  for (i1 = 1; i1 <= tunptr; i1++) {
    cave[tunstk[i1].y][tunstk[i1].x].fval  = corr_floor1.ftval;
    cave[tunstk[i1].y][tunstk[i1].x].fopen = corr_floor1.ftopen;
  }

  for (i1 = 1; i1 <= wallptr; i1++) {
    //with cave[wallstk[i1].y][wallstk[i1].x]. do;
    if (cave[wallstk[i1].y][wallstk[i1].x].fval == 9) {
      if (randint(100) < DUN_TUN_PEN) {
	place_door(wallstk[i1].y,wallstk[i1].x);
      } else {
	cave[wallstk[i1].y][wallstk[i1].x].fval  = corr_floor2.ftval;
	cave[wallstk[i1].y][wallstk[i1].x].fopen = corr_floor2.ftopen;
      }
    }
  }
};
//////////////////////////////////////////////////////////////////////
static boolean next_to(integer y, integer x)
{
  obj_set four_five_six = {4,5,6};
  boolean return_value = false;

  if (next_to8(y,x,four_five_six) > 2) {
    if (is_in(cave[y-1][x].fval, wall_set) &&
	is_in(cave[y+1][x].fval, wall_set)) {
      return_value = true;
    } else if (is_in(cave[y][x-1].fval, wall_set) &&
	       is_in(cave[y][x+1].fval, wall_set)) {
      return_value = true;
    } else {
      return_value = false;
    }
  } else {
    return_value = false;
  }

  return return_value;
};
//////////////////////////////////////////////////////////////////////
static void try_door(integer y, integer x)
{
  /*{ Places door at y,x position if at least 2 walls found }*/
 
  if (randint(100) > DUN_TUN_JCT) {
    if (cave[y][x].fval == corr_floor1.ftval) {
      if (next_to(y,x)) {
	place_door(y,x);
      }
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void cave_gen()
{
  /*{ Cave logic flow for generation of new dungeon }*/

  boolean      room_map[21][21];
  integer      i1,i2,i3,i4,y1,x1,y2,x2,pick1,pick2,row_rooms,col_rooms;
  integer      alloc_level,bx,by,qty;
  worlint      yloc[401], xloc[401]; //  : array [1..400] of worlint;
  boolean      bm_found;
 
  obj_set just_four    = {4};
  obj_set one_two      = {1,2};
  obj_set one_two_four = {1,2,4};

  set_seed(get_seed());
  row_rooms = 2*trunc(cur_height/SCREEN_HEIGHT);
  col_rooms = 2*trunc(cur_width /SCREEN_WIDTH);

  for (i1 = 1; i1 <= row_rooms; i1++) {
    for (i2 = 1; i2 <= col_rooms; i2++) {
      room_map[i1][i2] = false;
    }
  }

  qty = randnor(DUN_ROO_MEA,2);
  for (i1 = 1; i1 <= qty; i1++) {
    room_map[randint(row_rooms)][randint(col_rooms)] = true;
  }

  i3 = 0;

  for (i1 = 1; i1 <= row_rooms; i1++) {
    for (i2 = 1; i2 <= col_rooms; i2++) {
      if (room_map[i1][i2] == true) {
	i3 = i3 + 1;
	yloc[i3] = (i1-1)*(quart_height*2 + 1) + quart_height + 1;
	xloc[i3] = (i2-1)*(quart_width*2  + 1) + quart_width  + 1;
	if (dun_power > randint(DUN_UNUSUAL)) {
	  switch (randint(5)) {
	  case 1 : build_type1(yloc[i3],xloc[i3]); break;
	  case 2 : build_type2(yloc[i3],xloc[i3]); break;
	  case 3 : build_type3(yloc[i3],xloc[i3]); break;
	  case 4 : build_type4(yloc[i3],xloc[i3]); break;
	  case 5 : build_type5(yloc[i3],xloc[i3]); break;
	  }
	} else {
	  build_room(yloc[i3],xloc[i3]);
	}
      }
    }
  }
  
  for (i4 = 1; i4 <= i3; i4++) {
    pick1 = randint(i3);
    pick2 = randint(i3);
    y1 = yloc[pick1];
    x1 = xloc[pick1];
    yloc[pick1] = yloc[pick2];
    xloc[pick1] = xloc[pick2];
    yloc[pick2] = y1;
    xloc[pick2] = x1;
  }
  
  doorptr = 0;
  for (i4 = 1; i4 <= i3-1; i4++) {
    y1 = yloc[i4];
    x1 = xloc[i4];
    y2 = yloc[i4+1];
    x2 = xloc[i4+1];
    tunnel(y2,x2,y1,x1);
  }
  
  fill_cave(rock_wall1);
  for (i1 = 1; i1 <= DUN_STR_MAG; i1++) {
    place_streamer(rock_wall2,DUN_STR_MC);
  }
  
  for (i1 = 1; i1 <= DUN_STR_QUA; i1++) {
    place_streamer(rock_wall3,DUN_STR_QC);
  }
  
  place_boundry();
  /*{ Place intersection doors }*/
  for (i1 = 1; i1 <= doorptr; i1++) {
    try_door(doorstk[i1].y,doorstk[i1].x-1);
    try_door(doorstk[i1].y,doorstk[i1].x+1);
    try_door(doorstk[i1].y-1,doorstk[i1].x);
    try_door(doorstk[i1].y+1,doorstk[i1].x);
  }
  
  alloc_level = dun_power div 3;
  if (alloc_level < 2) {
    alloc_level = 2;
  } else if (alloc_level > 10) {
    alloc_level = 10;
  }
  
  if (dun_level < townlist[town_num].max_depth) {
    place_stairs(2,randint(3)+2,3);
  }
  
  place_stairs(1,randint(3)+2,3);
  alloc_monster(one_two,(randint(8)+MIN_MALLOC_LEVEL+alloc_level),0,true);
  alloc_object(just_four,3,randint(alloc_level));
  alloc_object(one_two,5,randnor(TREAS_ROOM_ALLOC,3));
  alloc_object(one_two_four,5,randnor(TREAS_ANY_ALLOC,3));
  alloc_object(one_two_four,4,randnor(TREAS_GOLD_ALLOC,3));
  alloc_object(one_two_four,1,randint(alloc_level));
  
  if ((py.misc.lev > 5) && (randint(5) == 1)) { 
    /*{Requirement for BM to appear -mwk}*/
    
    bm_found = false;
    while (!bm_found) {
      by = randint(MAX_HEIGHT);
      bx = randint(MAX_WIDTH);
      if (cave[by][bx].fopen) {
	place_trap(by,bx,3,0);
	do_black_market();
	bm_found = true;
      }
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void special_cave_gen(integer number)
{
  /*{ Logic flow for generation of new *special* dungeons - 200,250 meters}*/

  integer   alloc_level,i1,i2,rx,ry;
  integer   row_dir,col_dir;

  obj_set one_two      = {1,2};

  set_seed(get_seed());

  switch (number) {
  case 1: 
    for (i1 = 1; i1 <= cur_height; i1++) {
      for (i2 = 1; i2 <= cur_width; i2++) {
	cave[i1][i2].pl = true;
	if (randint(5) == 1) {
	  cave[i1][i2].fval  = rock_wall1.ftval;
	  cave[i1][i2].fopen = rock_wall1.ftopen;
	} else {
	  cave[i1][i2].fval = lopen_floor.ftval;
	}
      }
    }
    break;

  case 2:
    for (i1 = 1; i1 <= cur_height; i1++) {
      for (i2 = 1; i2 <= cur_width; i2++) {
	cave[i1][i2].fval  = dopen_floor.ftval;
	cave[i1][i2].fopen = dopen_floor.ftopen;
	if (randint(10) > 3) {  /*{make dungeon 70% lit}*/
	  cave[i1][i2].pl  = true;
	}
      }
    }

    for (i1 = 1; i1 <= 90; i1++) {
      ry = randint(cur_height-2)+1;	
      rx = randint(cur_width-2)+1;
      for (i2 = 1; i2 <= 60; i2++) {
	rand_dir(&row_dir,&col_dir,0,0,0,0,4);
	if (in_bounds(ry+row_dir,rx+col_dir)) {
	  ry = ry + row_dir;
	  rx = rx + col_dir;
	  cave[ry][rx].fval  = rock_wall2.ftval;
	  cave[ry][rx].fopen = rock_wall2.ftopen;
	}
      }
    }
    break;

  case 3:
    fill_cave(rock_wall3);
    for (i1 = 1; i1 <= 300; i1++) {
      ry = randint(cur_height-2)+1;	
      rx = randint(cur_width-2)+1;
      for (i2 = 1; i2 <= 60; i2++) {
	rand_dir(&row_dir,&col_dir,0,0,0,0,4);
	if (in_bounds(ry+row_dir,rx+col_dir)) {
	  ry = ry + row_dir;
	  rx = rx + col_dir;
	  cave[ry][rx].fval  = dopen_floor.ftval;
	  cave[ry][rx].fopen = dopen_floor.ftopen;
	}
      }
    }
    break;
  } /*{case}*/

  place_boundry();
  alloc_level = dun_power div 3;

  if (alloc_level < 2) {
    alloc_level = 2;
  } else if (alloc_level > 10) {
    alloc_level = 10;
  }

  if (dun_level < townlist[town_num].max_depth) {
    place_stairs(2,randint(5)+5,3);
  }

  place_stairs(1,randint(5)+5,3);
  alloc_monster(one_two,(randint(40)+MIN_MALLOC_LEVEL+alloc_level),0,true);
  alloc_object(one_two,5,randnor(TREAS_ROOM_ALLOC,3));
  alloc_object(one_two,5,randnor(TREAS_ANY_ALLOC,3));
  alloc_object(one_two,4,randnor(TREAS_GOLD_ALLOC,3));
  alloc_object(one_two,1,randint(alloc_level));
};
//////////////////////////////////////////////////////////////////////
static void bottom_cave_gen()
{
  /*{Logic Flow for special "Bottom of Cave" type.}*/ 

  boolean       room_map[21][21];
  integer       i1,i2,i3,row_rooms,col_rooms;
  integer       qty;
  worlint       yloc[401], xloc[401]; // : array [1..400] of worlint;

  obj_set       just_one    = {1};
  obj_set       eight_nine  = {8,9};

  set_seed(get_seed());
  row_rooms = 2*trunc(cur_height/SCREEN_HEIGHT);
  col_rooms = 2*trunc(cur_width /SCREEN_WIDTH);

  for (i1 = 1; i1 <= row_rooms; i1++) {
    for (i2 = 1; i2 <= col_rooms; i2++) {
      room_map[i1][i2] = false;
    }
  }

  qty = randnor(DUN_ROO_MEA,2);
  for (i1 = 1; i1 <= qty; i1++) {
    room_map[randint(row_rooms)][randint(col_rooms)] = true;
  }

  i3 = 0;

  for (i1 = 1; i1 <= row_rooms; i1++) {
    for (i2 = 1; i2 <= col_rooms; i2++) {
      if (room_map[i1][i2] == true) {
	i3 = i3 + 1;
	yloc[i3] = (i1-1)*(quart_height*2 + 1) + quart_height + 1;
	xloc[i3] = (i2-1)*(quart_width*2  + 1) + quart_width  + 1;
	switch (randint(5)) {
	case 1 : build_type1(yloc[i3],xloc[i3]); break;
	case 2 : build_type2(yloc[i3],xloc[i3]); break;
	case 3 : build_type3(yloc[i3],xloc[i3]); break;
	case 4 : build_type4(yloc[i3],xloc[i3]); break;
	case 5 : build_type5(yloc[i3],xloc[i3]); break;
	}
      }
    }
  }
  
  place_boundry();

  for (i1 = 1; i1 <= cur_height; i1++) {
    for (i2 = 1; i2 <= cur_width; i2++) {
      //with cave[i1][i2]. do;
      if ((cave[i1][i2].fval == rock_wall1.ftval) && (randint(10) == 1)) {
	place_secret_door(i1,i2);
      } else if (cave[i1][i2].fval == 0) {
	cave[i1][i2].fval = dopen_floor.ftval;  /*{tval=1}*/
	cave[i1][i2].fopen = dopen_floor.ftopen;  
      } else if (is_in(cave[i1][i2].fval,eight_nine)) {
	
	cave[i1][i2].fval = rock_wall1.ftval;
	cave[i1][i2].fopen = rock_wall1.ftopen;
      }
    }
  }
  
  if (dun_level < townlist[town_num].max_depth) {
    place_stairs(2,randint(3)+5,3);
  }
  
  place_stairs(1,randint(3)+5,3);
  alloc_monster(just_one,(randint(20)+MIN_MALLOC_LEVEL),0,true);

  if (dun_power == WIN_MON_APPEAR) {
    if (!(is_boss_dead || out_of_time)) {
      place_the_boss();
    }
  }

  if ((dun_level == 10) && (!(defeated_badguy))) {
    place_badguy_monster(town_num);
  }
};
//////////////////////////////////////////////////////////////////////
static void build_store(integer store_num, integer y, integer x,
			integer num_in_row)
{
  /*{ Town  ::  Builds a store at a row,column coordinate }*/

  integer yval,y_height,y_depth,xval,x_adjst,x_left,x_right,i1,i2,cur_pos;

  yval     = (y-1)*6 + 5;
  xval     = (x-1)*(40-6*num_in_row) + (21-3*num_in_row);
  /*{ this formula yields xval=(x-1)*22+11 if 3 stores in the row
    and xval=(x-1)*16+8 for 4 stores in the row. }*/
  x_adjst  = 6 - (num_in_row - 3)*3;
  y_height = yval - randint(2);
  y_depth  = yval + randint(2);
  x_left   = xval - 1 - randint(x_adjst);
  x_right  = xval + 1 + randint(x_adjst);


  for (i1 = y_height; i1 <= y_depth; i1++) {
    for (i2 = x_left; i2 <= x_right; i2++) {
      cave[i1][i2].fval  = boundry_wall.ftval;
      cave[i1][i2].fopen = boundry_wall.ftopen;
    }
  }


  /*{ determines which wall and where to put the door }*/
  switch (randint(4)) {
  case 1 : 
    i1 = randint(y_depth-y_height) + y_height - 1;
    i2 = x_left;
    break;

  case 2 : 
    i1 = randint(y_depth-y_height) + y_height - 1;
    i2 = x_right;
    break;

  case 3 : 
    i1 = y_depth;
    i2 = randint(x_right-x_left) + x_left - 1;
    break;

  case 4 : 
    i1 = y_height;
    i2 = randint(x_right-x_left) + x_left - 1;
    break;
  }

  //with cave[i1][i2]. do;
  cave[i1][i2].fval  = corr_floor3.ftval;
  cave[i1][i2].fopen = corr_floor3.ftopen;
  popt(&cur_pos);
  cave[i1][i2].tptr = cur_pos;
  t_list[cur_pos] = store_door[store_num];
};
//////////////////////////////////////////////////////////////////////
static void town_gen()
{
  /*{ Town logic flow for generation of new town }*/


  integer    i1,i2,i3,i4,i5;
  integer    rooms[MAX_DOORS+1]; //: array [1..max_doors] of integer;
  obj_set    one_two      = {1,2};
 
  //set_seed(town_seed);
  set_seed(town_num);

  for (i1 = 1; i1 <= MAX_DOORS; i1++) {
    rooms[i1] = i1;
  }

  i4 = MAX_DOORS;      /*{i4 is store counter}*/

  for (i1 = 1; i1 <= 3; i1++) {    /*{y value}*/
    for (i2 = 1; i2 <= 4; i2++) {  /*{x value}*/
      i3 = randint(i4);
      build_store(rooms[i3],i1,i2,4); /*{4 is number in row}*/
      for (i5 = i3; i5 <= i4-1; i5++) {
	rooms[i5] = rooms[i5+1];
      }
      i4 = i4 - 1;
    }
  }

  fill_cave(dopen_floor);
  place_boundry();

  if ((turn < 4320) || (turn > 12960)) {
    /*{Night}*/
    for (i1 = 1; i1 <= cur_height; i1++) {
      for (i2 = 1; i2 <= cur_width; i2++) {
	if (cave[i1][i2].fval != dopen_floor.ftval) {
	  cave[i1][i2].pl = true;
	}
      }
    }
    place_stairs(2,3,0);
    set_seed(get_seed());
    alloc_monster(one_two,MIN_MALLOC_TN,3,true);

  } else {
    /*{ Day }*/
    for (i1 = 1; i1 <= cur_height; i1++) {
      for (i2 = 1; i2 <= cur_width; i2++) {
	cave[i1][i2].pl = true;
      }
    }
    place_stairs(2,3,0);
    set_seed(get_seed());
    alloc_monster(one_two,MIN_MALLOC_TD,3,true);
  }
};
//////////////////////////////////////////////////////////////////////
static void place_building_door(integer spoty, integer spotx, integer door_num)
{
  integer cur_pos;

  //with cave[spoty][spotx]. do;
  cave[spoty][spotx].fval  = corr_floor3.ftval;
  cave[spoty][spotx].fopen = corr_floor3.ftopen;
  popt(&cur_pos);
  cave[spoty][spotx].tptr  = cur_pos;
  t_list[cur_pos] = building_door[door_num];
};
//////////////////////////////////////////////////////////////////////
static void building_gen()
{
  /*{ Logic flow for generation of tall building }*/
  
  integer   i1,i2;
  
  fill_cave(lopen_floor);
  place_boundry();
  for (i2 = 1; i2 <= 30; i2++) {
    //with cave[5][i2]. do;
    cave[5][i2].fval  = boundry_wall.ftval;
    cave[5][i2].fopen = boundry_wall.ftopen;
  }

  for (i1 = 2; i1 <= 9; i1++) {
    for (i2 = 2; i2 <= 7; i2++) {
      //with cave[i1][i2]. do;
      cave[i1][i2].fval  = boundry_wall.ftval;
      cave[i1][i2].fopen = boundry_wall.ftopen;
    }
  }    

  for (i2 = 1; i2 <= 30; i2++) {
    for (i2 = 23; i2 <= 29; i2++) {
      //with cave[i1][i2]. do;
      cave[i1][i2].fval  = boundry_wall.ftval;
      cave[i1][i2].fopen = boundry_wall.ftopen;
    }
  }

  place_building_door(7,7,1);  /*{Hotel}*/
  place_building_door(7,23,2); /*{Bank}*/
  place_building_door(3,7,3);  /*{Clinic}*/
  place_building_door(3,23,4); /*{NewsInc}*/
  place_closed_door(5,15);
  place_down_stairs(9,15);
  char_row =  8;
  char_col = 15;
};
//////////////////////////////////////////////////////////////////////
void generate_cave()
{
    /*{ Generates a random dungeon level }*/
  //coords   doorstk[100+1]; // : array [1..100] of coords;
  //integer  doorptr;
    
    panel_row_min     =  0;
    panel_row_max     =  0;
    panel_col_min     =  0;
    panel_col_max     =  0;
    char_row          = -1;
    char_col          = -1;
    tlink();
    mlink();
    blank_cave();
 
    if (dun_level < 0) {
	cur_height     = 10;
	cur_width      = 30;
	max_panel_rows = 0;
	max_panel_cols = 0;
	panel_row      = max_panel_rows;
	panel_col      = max_panel_cols;
	dun_power      = 0;
	building_gen();
    } else if (dun_level == 0) {
	cur_height     = SCREEN_HEIGHT;
	cur_width      = SCREEN_WIDTH;
	max_panel_rows = trunc(cur_height/SCREEN_HEIGHT)*2 - 2;
	max_panel_cols = trunc(cur_width /SCREEN_WIDTH )*2 - 2;
	panel_row      = max_panel_rows;
	panel_col      = max_panel_cols;
	dun_power      = 0;
	town_gen();
	if (exited_office) {
	    char_row   = office_char_row;
	    char_col   = office_char_col;
	    exited_office = false;
	};
    } else {
	cur_height     = MAX_HEIGHT;
	cur_width      = MAX_WIDTH;
	max_panel_rows = trunc(cur_height/SCREEN_HEIGHT)*2 - 2;
	max_panel_cols = trunc(cur_width /SCREEN_WIDTH )*2 - 2;
	panel_row      = max_panel_rows;
	panel_col      = max_panel_cols;
	dun_power      = dun_level + townlist[town_num].lev_plus;
	switch (dun_level % 10) {
	case 0:            bottom_cave_gen();    break;
	case 5:            special_cave_gen(1);  break;
	case 3: case 8:    special_cave_gen(2);  break;
	case 2: case 7:    special_cave_gen(3);  break;
	default:           cave_gen();           break;
	}
    }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
/* END FILE generate.c */
