/* main.c */
/**/

#include "boss.h"
#include "main.h"

/* Look!  More globals!!! */
boolean    moria_flag;      /*{ Next level when true  }*/
boolean    reset_flag;      /*{ Do not move creatures }*/
boolean    search_flag;     /*{ Player is searching   }*/
boolean    teleport_flag;   /*{ Handle telport traps  }*/
boolean    player_light;    /*{ Player carrying light }*/

boolean    unleashed_doom;  /*{ The Doom is still buggy }*/
integer    doomy,doomx;

integer    old_chp,old_cmana; /*{ Detect changes } */

real       regen_amount;      /*{ Regenerate hp and mana}*/

boolean    save_msg_flag;     /*{ Msg flag after INKEY}*/

boolean    cmd_md = 0;

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void panel_bounds()
{
  /*{ Calculates current boundries				-RAK-	}*/

  panel_row_min = (trunc(panel_row*(SCREEN_HEIGHT/2)) + 1);
  panel_row_max = panel_row_min + SCREEN_HEIGHT - 1;

  panel_col_min = (trunc(panel_col*(SCREEN_WIDTH/2)) + 1);
  panel_col_max = panel_col_min + SCREEN_WIDTH - 1;

  panel_row_prt = panel_row_min - 2;
  panel_col_prt = panel_col_min - 15;

}
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
boolean no_light()
{
  /*{ Returns true if player has no light                   -RAK-   }*/

  boolean    return_value = false;
  
  //with cave[char_row,char_col] do;
  if (!(cave[char_row][char_col].tl)) {
    if (!(cave[char_row][char_col].pl)) {
      return_value = true;
    }
  }
  return return_value;
}
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
boolean get_panel(integer y, integer x, boolean forceit)
{
  /*{ Given an row (y) and col (x), this routine detects  -RAK-     }*/
  /*{ when a move off the screen has occurred and figures new borders}*/
  /* forceit forcses the panel bounds to be recalculated (show_location). */

  integer   prow,pcol;
  boolean   return_value;

  prow = panel_row;
  pcol = panel_col;

  if (forceit || (y < panel_row_min + 2) || (y > panel_row_max - 2)) {
    prow = trunc((y - 2)/(SCREEN_HEIGHT/2));
    if (prow > max_panel_rows) {
      prow = max_panel_rows;
    } else if (prow < 0) {
      prow = 0;
    }
  }

  if (forceit || (x < panel_col_min + 3) || (x > panel_col_max - 3)) {
    pcol = trunc((x - 3)/(SCREEN_WIDTH/2));
    if (pcol > max_panel_cols) {
      pcol = max_panel_cols;
    } else if (pcol < 0) {
      pcol = 0;
    }
  }

  if ((prow != panel_row) || (pcol != panel_col) || !(cave_flag)) {
    panel_row = prow;
    panel_col = pcol;
    panel_bounds();
    cave_flag = true;
    return_value = true;
  } else {
    return_value = false;
  }

  return return_value;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
static void make_boss_stronger()
{
  integer   num;

  //with c_list[max_creatures]. do;
  num = total_winner_num + 1;
  sprintf(c_list[max_creatures].name, "Boss %ld", num);
  c_list[max_creatures].ac    += 10;
  c_list[max_creatures].mexp  += 2000;
  c_list[max_creatures].speed += 2;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void rest_off()
{
  py.flags.rest    = 0;
  py.flags.status &= 0xFFFFFDFF;
  erase_line(1,1);
  prt_rest();
  //with py.flags do;
  PF.food_digested++;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void midnight()
{
  /*{The Witching Hour!!}*/
  turn -= 17280;
  day++;
  act_bal   += trunc(act_bal * iod_val / 7);
  principal += trunc(principal * iol_val / 7);
  store_maint();
  if (day == DAY_LIMIT) {
    time_out();
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
static void regenhp(real percent)
{
  /*{ Regenerate hit points }*/

  //with py.misc do;
  PM.chp += PM.mhp*percent + PLAYER_REGEN_HPBASE;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
static void regenmana(real percent)
{
  /*{ Regenerate mana points }*/

  //with py.misc do;
  PM.cmana += PM.mana*percent + PLAYER_REGEN_MNBASE;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void teleport(integer dis)
{
  /*{ Teleport the player to a new location }*/

  integer y,x;
  integer i1, i2;

  do {
    y = randint(cur_height);
    x = randint(cur_width);
    while (distance(y,x,char_row,char_col) > dis) {
      y += trunc((char_row-y)/2);
      x += trunc((char_col-x)/2);
    }
  } while (!((cave[y][x].fopen) && (cave[y][x].cptr  < 2)));

  move_rec(char_row,char_col,y,x);

  for (i1 = char_row-1; i1 <= char_row+1; i1++) {
    for (i2 = char_col-1; i2 <= char_col+1; i2++) {
      //with cave[i1][i2]. do;
      cave[i1][i2].tl = false;
      if (!(test_light(i1,i2))) {
	unlite_spot(i1,i2);
      }
    }
  }
  
  if (test_light(char_row,char_col)) {
    lite_spot(char_row,char_col);
  }

  char_row = y;
  char_col = x;
  move_char(5);
  creatures(false);
  teleport_flag = false;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
boolean test_hit(integer bth, integer level, integer pth, integer ac)
{
  /*{ Attacker's level and pluses, defender's AC }*/
  integer i1;
  boolean return_value = false;

  if (search_flag) {
    search_off();
  }
  if (py.flags.rest > 0) {
    rest_off();
  }

  i1 = bth + level*BTH_LEV_ADJ + pth*BTH_PLUS_ADJ;

  if (randint(i1) > ac) {             /*{ Normal hit }*/
    return_value = true;
  } else if (randint(BTH_HIT) == 1) { /*{Automatic Hit}*/
    return_value = true;
  } else {                            /*{ Missed }*/
    return_value = false;
  }

  return return_value;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void take_hit(integer damage, char *hit_from)
{
  /*{ Decreases players hit points and sets death flag if neccessary}*/

  if (py.flags.invuln > 0) {
    damage /= 2;
  }
  py.misc.chp -= damage;
  if (search_flag) { search_off() ; }
  if (py.flags.rest > 0) { rest_off(); }
  
  flush();
  
  if (py.misc.chp <= -1) {
    if (!death) {
      /*{ Hee, hee... Ain't I mean? }*/
      death = true;
      strcpy(died_from, hit_from);
    }
    moria_flag = true;
  } else {
    prt_chp();
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void change_speed(integer num)
{
  /*
    { Changes speed of monsters relative to player
    Note: When the player is sped up or slowed down, I simply
    change the speed of all the monsters.  This greatly
    simplified the logic.}
  */

  integer i1;

  py.flags.speed += num;
  i1 = muptr;

  while (i1 != 0) {
    m_list[i1].cspeed += num;
    i1 = m_list[i1].nptr;
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void py_bonuses(treasure_type *tobj, integer factor)
{
  /* 
     { Player bonuses
     When an item is worn or taken off, this re-adjusts the player
     bonuses.  Factor=1 : wear; Factor=-1 : removed }
  */

  unsigned   item_flags;
  integer    i1,old_dis_ac;

  //with py.flags do;
  if (PF.slow_digest) {
    PF.food_digested += 1;
  }
  if (PF.regenerate) {
    PF.food_digested -= 3;
  }
  PF.see_inv     = false;
  PF.teleport    = false;
  PF.free_act    = false;
  PF.slow_digest = false;
  PF.aggravate   = false;
  PF.sustain_str = false;
  PF.sustain_int = false;
  PF.sustain_wis = false;
  PF.sustain_con = false;
  PF.sustain_dex = false;
  PF.sustain_chr = false;
  PF.resist_gas  = false;
  PF.fire_resist = false;
  PF.acid_resist = false;
  PF.cold_resist = false;
  PF.regenerate  = false;
  PF.lght_resist = false;
  PF.ffall       = false;
  
  if (uand(0x00000001,tobj->flags) != 0) {
    change_stat(&py.stat.cstr,tobj->p1,factor);
    change_stat(&py.stat.str,tobj->p1,factor);
    print_stat |= 0x0001;
  }

  if (uand(0x00000002,tobj->flags) != 0) {
    change_stat(&py.stat.cdex,tobj->p1,factor);
    change_stat(&py.stat.dex,tobj->p1,factor);
    print_stat |= 0x0002;
  }
  
  if (uand(0x00000004,tobj->flags) != 0) {
    change_stat(&py.stat.ccon,tobj->p1,factor);
    change_stat(&py.stat.con,tobj->p1,factor);
    print_stat |= 0x0004;
  }
  
  if (uand(0x00000008,tobj->flags) != 0) {
    change_stat(&py.stat.cint,tobj->p1,factor);
    change_stat(&py.stat.int_,tobj->p1,factor);
    print_stat |= 0x0008;
  }
  
  if (uand(0x00000010,tobj->flags) != 0) {
    change_stat(&py.stat.cwis,tobj->p1,factor);
    change_stat(&py.stat.wis,tobj->p1,factor);
    print_stat |= 0x0010;
  }
  
  if (uand(0x00000020,tobj->flags) != 0) {
    change_stat(&py.stat.cchr,tobj->p1,factor);
    change_stat(&py.stat.chr,tobj->p1,factor);
    print_stat |= 0x0020;
  }
  
  if (uand(0x00000040,tobj->flags) != 0) {
    py.misc.srh  += tobj->p1 * 10 * factor;
    py.misc.perc += tobj->p1 * 10 * factor;
  }
  
  if (uand(0x00000100,tobj->flags) != 0) {
    py.misc.stl += 2*factor;
  }
  
  if (uand(0x00001000,tobj->flags) != 0) {
    i1 = tobj->p1*factor;
    change_speed(-i1);
  }
  
  if (uand(0x08000000,tobj->flags) != 0) {
    if (factor > 0) {
      py.flags.blind += 1000;
    }
  }
  
  if (uand(0x10000000,tobj->flags) != 0) {
    if (factor > 0) {
      py.flags.afraid += 50;
    }
  }
  
  if (uand(0x40000000,tobj->flags) != 0) {
    py.flags.see_infra += (tobj->p1 * factor);
  }
  
  //with py.misc do;
  old_dis_ac    = PM.dis_ac;
  PM.ptohit     = tohit_adj(); /*{ Real To Hit    }*/
  PM.ptodam     = todam_adj(); /*{ Real To Dam    }*/
  PM.ptoac      = toac_adj();  /*{ Real To AC     }*/
  PM.pac        = 0;           /*{ Real AC        }*/
  PM.dis_th     = PM.ptohit;   /*{ Display To Hit }*/
  PM.dis_td     = PM.ptodam;   /*{ Display To Dam }*/
  PM.dis_ac     = 0;           /*{ Display To AC  }*/
  PM.dis_tac    = PM.ptoac;    /*{ Display AC     }*/
  
  for (i1 = 23; i1 <= EQUIP_MAX-1; i1++) {
    //with equipment[i1]. do;
    if (equipment[i1].tval > 0) {
      if (uand(0x80000000,equipment[i1].flags) == 0) {
	PM.pac    += equipment[i1].ac;
	PM.dis_ac += equipment[i1].ac;
      }
      
      PM.ptohit += equipment[i1].tohit;
      PM.ptodam += equipment[i1].todam;
      PM.ptoac  += equipment[i1].toac;
      
      if (strchr(equipment[i1].name,'^') == NULL) {
	PM.dis_th  += equipment[i1].tohit;
	PM.dis_td  += equipment[i1].todam;
	PM.dis_tac += equipment[i1].toac;
      }
    }
  }
  
  PM.dis_ac += PM.dis_tac;
  
  /*{ Add in temporary skill increases }*/
  //with py.flags. do;
  if (PF.blessed > 0) {
    PM.pac    += 2;
    PM.dis_ac += 2;
  }
  if (PF.detect_inv > 0) {
    PF.see_inv = true;
  }
  
  if (old_dis_ac != PM.dis_ac) {
    print_stat |= 0x0040;
  }
  
  item_flags = 0;
  for (i1 = 23; i1 <= EQUIP_MAX-1; i1++) {
    //with equipment[i1]. do;
    item_flags |= equipment[i1].flags;
    
    if (uand(0x00400000,equipment[i1].flags) != 0) {
      switch (equipment[i1].p1) {
      case 1 : py.flags.sustain_str = true; break;
      case 2 : py.flags.sustain_int = true; break;
      case 3 : py.flags.sustain_wis = true; break;
      case 4 : py.flags.sustain_con = true; break;
      case 5 : py.flags.sustain_dex = true; break;
      case 6 : py.flags.sustain_chr = true; break;
      default: break;
      }
    }
  }
  
  if (uand(0x00000080,item_flags) != 0) {
    py.flags.slow_digest = true;
  }
  if (uand(0x00000200,item_flags) != 0) {
    py.flags.aggravate = true;
  }
  if (uand(0x00000400,item_flags) != 0) {
    py.flags.teleport = true;
  }
  if (uand(0x00000800,item_flags) != 0) {
    py.flags.regenerate = true;
  }
  if (uand(0x00040000,item_flags) != 0) {
    py.flags.resist_gas = true;
  }
  if (uand(0x00080000,item_flags) != 0) {
    py.flags.fire_resist = true;
  }
  if (uand(0x00100000,item_flags) != 0) {
    py.flags.acid_resist = true;
  }
  if (uand(0x00200000,item_flags) != 0) {
    py.flags.cold_resist = true;
  }
  if (uand(0x00800000,item_flags) != 0) {
    py.flags.free_act = true;
  }
  if (uand(0x01000000,item_flags) != 0) {
    py.flags.see_inv = true;
  }
  if (uand(0x02000000,item_flags) != 0) {
    py.flags.lght_resist = true;
  }
  if (uand(0x04000000,item_flags) != 0) {
    py.flags.ffall = true;
  } 
  
  //with py.flags do;
  if (PF.slow_digest) {
    PF.food_digested -= 1;
  }
  if (PF.regenerate) {
    PF.food_digested += 3;
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void lite_spot(integer y,integer x)
{
  /*{ Lights up given location }*/
  if (panel_contains(y,x)) {
    print(loc_symbol(y,x),y,x);
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void unlite_spot(integer y,integer x)
{
  /*{ Blanks out given location }*/

  if (panel_contains(y,x)) {
    print(' ',y,x);
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void move_rec(integer y1,integer x1,integer y2,integer x2)
{
  /*{ Moves creature record from one space to another }*/
  if ((y1 != y2) || (x1 != x2)) {
    cave[y2][x2].cptr = cave[y1][x1].cptr;
    cave[y1][x1].cptr = 0;
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
byteint change_stat(byteint *stat, integer  amount, integer factor)
{
  /*{ Changes stats up or down for magic items }*/

  integer  i1,i2,i3;

  i2 = amount * factor;
  i3 = abs(amount);

  for (i1 = 1; i1 <= i3; i1++) {
    if (i2 < 0) {
      *stat = de_statt(*stat);
    } else {
      *stat = in_statt(*stat);
    }
  }

  return *stat;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
boolean panel_contains(integer y, integer x)
{
  /*{ Tests a given point to see if it is within the screen -RAK-   }*/
  /*{ boundries.                                                    }*/

  boolean   return_value = false;

  if ((y >= panel_row_min) && (y <= panel_row_max)) {
    if ((x >= panel_col_min) && (x <= panel_col_max)) {
      return_value = true;
    }
  }

  return return_value;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void light_dam(integer dam, char *kb_str)
{
  /*{ Lightning bolt the sucker away.}*/

  if (py.flags.lght_resist) {
    dam /= 2;
  }
  take_hit(dam,kb_str);
  print_stat |= 0x0080;
}; 
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void acid_dam(integer dam, char *kb_str) 
{
  /*{ Throw acid on the hapless victim }*/
  
  integer  flag = 0;
  obj_set  acid_says_yum = {1,2,11,12,20,21,22,30,31,32,36};

  if (minus_ac(0x00100000)) {
    flag = 1;
  }
  if (py.flags.acid_resist) {
    flag += 2;
  }
  switch (flag) {
  case 0 : take_hit(dam,kb_str);               break;
  case 1 : take_hit(trunc(dam*0.75),kb_str);   break;
  case 2 : take_hit(trunc(dam*0.5),kb_str);    break;
  case 3 : take_hit(trunc(dam*0.25),kb_str);   break;
  }
  print_stat |= 0x00C0;
  if (inven_damage(acid_says_yum,3) > 0) {
    msg_print("There is an acrid smell coming from your pack!");
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void cold_dam(integer dam, char *kb_str) 
{
  /*{ Freeze him to death.}*/

  if (py.flags.cold_resist) {
    dam /= 2;
  }
  if (py.flags.resist_cold > 0) {
    dam /= 2;
  }
  take_hit(dam,kb_str);
  print_stat |= 0x0080;
  if (inven_damage(destroyed_by_cold,5) > 0) {
    msg_print("Something shatters inside your pack!");
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void fire_dam(integer dam, char *kb_str) 
{
  /*{ Burn the fool up.}*/
  obj_set burn_baby_burn = {12,20,21,22,30,31,32,36,55,70,71};

  if (py.flags.fire_resist) {
    dam /= 2;
  }
  if (py.flags.resist_heat > 0) {
    dam /= 2;
  }
  take_hit(dam,kb_str);
  print_stat |= 0x0080;
  if (inven_damage(burn_baby_burn,3) > 0) {
    msg_print("There is smoke coming from your pack!");
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void corrode_gas(integer dam, char *kb_str)
{
    /*{ Corrode the unsuspecting person's armor }*/

    obj_set mushy_pile_of_goo = {23,33,34,35,65};

    if (! (minus_ac(0x00100000))) {
	take_hit(dam,kb_str);
    }
    print_stat |= 0x0040;
    if (inven_damage(mushy_pile_of_goo,5) > 0) {
	msg_print("There is an acrid smell coming from your pack.");
    }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void poison_gas(integer dam, char *kb_str)
{
  /*{ Poison gas the idiot.}*/

  if (PF.resist_gas) {
    dam /= 2;
  }
  take_hit(dam,kb_str);
  print_stat |= 0x0040;
  py.flags.poisoned += 4 + randint(dam);
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
boolean delete_object(integer y, integer x)
{
  /*{ Deletes object from given location                    -RAK-   }*/

  boolean    return_value = false;

  //with cave[y,x] do;
  if (t_list[cave[y][x].tptr].tval == 109 ) { /* Secret_door */
    cave[y][x].fval = corr_floor3.ftval;
  }
  cave[y][x].fopen = true;
  pusht(cave[y][x].tptr);
  cave[y][x].tptr = 0;
  cave[y][x].fm = false;
  if (test_light(y,x)) {
    lite_spot(y,x);
    return_value = true;
  } else {
    unlite_spot(y,x);
  }

  return return_value;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
boolean minus_ac(integer typ_dam)
{
  /*{ AC gets worse
    Note: This routine affects magical AC bonuses so that stores
    can detect the damage. }
  */
  /*{ AC gets worse                                         -RAK-   }*/
  /*{ Note: This routine affects magical AC bonuse so that stores   }*/
  /*{       can detect the damage.                                  }*/

  integer   i1,i2;
  integer   tmp[9];  /*  : array [1..8] of integer;*/
  vtype     out_str,out_val;
  boolean   return_value = false;

  i1 = 0;
  if (equipment[26].tval > 0) {
    i1++;
    tmp[i1] = 26;
  }
  if (equipment[27].tval > 0) {
    i1++;
    tmp[i1] = 27;
  }
  if (equipment[32].tval > 0) {
    i1++;
    tmp[i1] = 32;
  }
  if (equipment[28].tval > 0) {
    i1++;
    tmp[i1] = 28;
  }
  if (equipment[24].tval > 0) {
    i1++;
    tmp[i1] = 24;
  }
  
  if (i1 > 0) {
    i2 = tmp[randint(i1)];
    //with equipment[i2] do;
    if ((equipment[i2].flags & typ_dam) != 0) {
      objdes(out_val,i2,false,'e');
      sprintf(out_str,"Your %s resists damage!",out_val);
      msg_print(out_str);
      return_value = true;
    } else if ((equipment[i2].ac + equipment[i2].toac) > 0) {
      objdes(out_val,i2,false,'e');
      sprintf(out_str,"Your %s is damaged!",out_val);
      msg_print(out_str);
      equipment[i2].toac--;
      py_bonuses(&blank_treasure,0);
      return_value = true;
    }
  }  

  return return_value;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void ml__draw_block(integer y1, integer x1, integer y2, integer x2)
{
  /*{ Given two sets of points, draw the block		}*/

  integer   i1,i2,xpos,xmax=0;
  integer   topp,bott,left,righ;
  integer   new_topp,new_bott,new_left,new_righ;
  vtype     floor_str,save_str;
  integer   floor_str_len = 0;
  char      tmp_char;
  boolean   flag;

  ENTER("ml__draw_block","m");

  /*{ From uppermost to bottom most lines player was on...  }*/
  /*{ Points are guaranteed to be on the screen (I hope...) }*/
  
//fprintf(debug_file,": draw_block: y1: %d  x1: %d  y2: %d  x2: %d\n",
//    y1,x1,y2,x2);
//fflush(debug_file);

  topp = maxmin(y1,y2,panel_row_min);
  bott = minmax(y1,y2,panel_row_max);
  left = maxmin(x1,x2,panel_col_min);
  righ = minmax(x1,x2,panel_col_max);
  
//fprintf(debug_file,": draw_block: topp: %d  bott: %d  left: %d  right: %d\n",
//      topp,bott,left,righ);
//fflush(debug_file);

  new_topp = y2 - 1;     /*{ Margins for new things to appear}*/
  new_bott = y2 + 1;
  new_left = x2 - 1;
  new_righ = x2 + 1;
  
  for (i1 = topp; i1 <= bott; i1++) {

    floor_str[0]  = 0;    /*{ Null out print string         }*/
    floor_str_len = 0;
    save_str[0]   = 0;
    xpos          = 0;

    for (i2 = left; i2 <= righ; i2++) { /*{ Leftmost to rightmost do}*/
      //with cave[i1,i2] do;
      if ((cave[i1][i2].pl) || (cave[i1][i2].fm)) {
	flag = (((i1==y1) && (i2==x1)) || ((i1==y2) && (i2==x2)));
	//flag = true;
      } else {
	flag = true;
	if (((i1 >= new_topp) && (i1 <= new_bott)) &&
	    ((i2 >= new_left) && (i2 <= new_righ))) {
	  if (cave[i1][i2].tl) {
	    if (is_in(cave[i1][i2].fval,pwall_set)) {
	      cave[i1][i2].pl = true;
	    } else if (cave[i1][i2].tptr > 0) {
	      if (is_in(t_list[cave[i1][i2].tptr].tval,light_set)) {
		if (!(cave[i1][i2].fm)) {
		  cave[i1][i2].fm = true;
		}
	      }
	    }
	  }
	}
      }
      
      if ((cave[i1][i2].pl) || (cave[i1][i2].tl) || (cave[i1][i2].fm)) {
	tmp_char = loc_symbol(i1,i2);
      } else {
	tmp_char = ' ';
      }
      
      if (py.flags.image > 0) {
	if (randint(12) == 1) {
	  tmp_char = (char)(randint(95) + 31);
	}
      }
      
      if (flag) {
	if (xpos == 0) { 
	  xpos = i2;
	}
	xmax = i2;
      }

      if (xpos > 0) {
	if (floor_str_len > 80) {
	  MSG(": ERROR draw_block floor_str_len too big\n");
	}	

	floor_str[floor_str_len++] = tmp_char;
      }
      
    } /* end for  i2 */
    
    if (floor_str_len > 80) {
      MSG(": ERROR draw_block floor_str_len too big\n");
    }	

    floor_str[floor_str_len] = 0;
//    fprintf(debug_file,":   floor before if %d: |%s|\n",i1,floor_str);
//    fflush(debug_file);
    
    if (xpos > 0) {
      i2 = i1;       /*{ Var for PRINT cannot be loop index}*/
      /*print(substr(floor_str,1,1+xmax-xpos),i2,xpos);*/

      if ( ((1+xmax-xpos+1) > 80)  ||  ((1+xmax-xpos+1) < 0) ) {
	MSG(": ERROR draw_block xmax-xpos is bad\n");
      }	

      floor_str[1+xmax-xpos+1] = 0;
      print_str(floor_str,i2,xpos);

    }
  }

  LEAVE("ml__draw_block","m");
};
//////////////////////////////////////////////////////////////////////
void ml__sub1_move_light(integer y1,integer x1,integer y2,integer x2)
{
  /*{ Normal movement                                   }*/

  integer   i1,i2;

  ENTER("ml__sub1_move_light","m");

  light_flag = true;

  for (i1 = y1-1; i1 <= y1+1; i1++) {   /*{ Turn off lamp light   }*/
    for (i2 = x1-1; i2 <= x1+1; i2++) {
      cave[i1][i2].tl = false;
    }
  }

  for (i1 = y2-1; i1 <= y2+1; i1++) {
    for (i2 = x2-1; i2 <= x2+1; i2++) {
      cave[i1][i2].tl = true;
    }
  }

  ml__draw_block(y1,x1,y2,x2);        /*{ Redraw area           }*/

  LEAVE("ml__sub1_move_light","m");
};
//////////////////////////////////////////////////////////////////////
void ml__sub2_move_light(integer y1,integer x1,integer y2,integer x2)
{
  /*{ When FIND_FLAG, light only permanent features     }*/

  integer   i1,i2,xpos;
  vtype     floor_str,save_str;
  integer   floor_str_len, save_str_len;
  char      tmp_char;
  boolean   flag;

  ENTER("ml__sub2_move_light","m");

  if (light_flag) {
    for (i1 = y1-1; i1 <= y1+1; i1++) {
      for (i2 = x1-1; i2 <= x1+1; i2++) {
	cave[i1][i2].tl = false;
      }
    }
    ml__draw_block(y1,x1,y1,x1);
    light_flag = false;
  }
  
  for (i1 = y2-1; i1 <= y2+1; i1++) {
    floor_str[0]  = 0;
    save_str[0]   = 0;
    floor_str_len = 0;
    save_str_len  = 0;
    xpos = 0;
    for (i2 = x2-1; i2 <= x2+1; i2++) {
      //with cave[i1,i2] do;
      flag = false;
      if (!((cave[i1][i2].fm) || (cave[i1][i2].pl))) {
	tmp_char = ' ';
	if (player_light) {
	  if (is_in(cave[i1][i2].fval, pwall_set)) {
	    cave[i1][i2].pl = true; /*{ Turn on perm light    }*/
	    tmp_char = loc_symbol(i1,i2);
	    flag = true;
	  } else {
	    if (cave[i1][i2].tptr > 0) {
	      if (is_in(t_list[cave[i1][i2].tptr].tval,light_set)) {
		cave[i1][i2].fm = true;  /*{ Turn on field marker  }*/
		tmp_char = loc_symbol(i1,i2);
		flag = true;
	      }
	    }
	  }
	}
      } else {
	tmp_char = loc_symbol(i1,i2);
      }
      
      if (flag) {
	if (xpos == 0) {
	  xpos = i2;
	}
	if (save_str[0] != 0) {
	  //floor_str := floor_str + save_str;
	  floor_str[floor_str_len] = 0;
	  save_str[save_str_len]   = 0;
	  strcat(floor_str, save_str);
	  floor_str_len += save_str_len;
	  save_str[0] = 0;
	  save_str_len = 0;
	}
	floor_str[floor_str_len++] = tmp_char;
      } else if (xpos > 0) {
	save_str[save_str_len++] = tmp_char;
      }
    } /* end for i2 */
    if (xpos > 0) {
      i2 = i1;
      floor_str[floor_str_len] = 0;
      print_str(floor_str,i2,xpos);
    }
  } /* end for i1 */

  LEAVE("ml__sub2_move_light","m");
};
//////////////////////////////////////////////////////////////////////
void ml__sub3_move_light(integer y1,integer x1,integer y2,integer x2)
{
  /*{ When blinded, move only the player symbol...              }*/

  integer  i1,i2;

  ENTER("ml__sub3_move_light","m");

  if (light_flag) {
    for (i1 = y1-1; i1 <= y1+1; i1++) {
      for (i2 = x1-1; i2 <= x1+1; i2++) {
	cave[i1][i2].tl = false;
      }
    }
    light_flag = false;
  }
  print(' ',y1,x1);
  print('@',y2,x2);

  LEAVE("ml__sub3_move_light","m");
};
//////////////////////////////////////////////////////////////////////
void ml__sub4_move_light(integer y1,integer x1,integer y2,integer x2)
{
  /*{ With no light, movement becomes involved...               }*/

  integer   i1,i2;
  
  ENTER("ml__sub4_move_light","m");

  light_flag = true;
  if (cave[y1][x1].tl) {
    for (i1 = y1-1; i1 <= y1+1; i1++) {
      for (i2 = x1-1; i2 <= x1+1; i2++) {
	cave[i1][i2].tl = false;
	if (test_light(i1,i2)) {
	  lite_spot(i1,i2);
	} else {
	  unlite_spot(i1,i2);
	}
      }
    }
  } else if (test_light(y1,x1)) {
    lite_spot(y1,x1);
  } else {
    unlite_spot(y1,x1);
  }
  print('@',y2,x2);

  LEAVE("ml__sub4_move_light","m");
};
//////////////////////////////////////////////////////////////////////
void move_light(integer y1,integer x1,integer y2,integer x2)
{
  /*{ Package for moving the character's light about the screen     }*/
  /*{ Three cases : Normal, Finding, and Blind              -RAK-   }*/

  if (py.flags.blind > 0) {
	ml__sub3_move_light(y1,x1,y2,x2);  /* blind */
  } else if (find_flag) {
	ml__sub2_move_light(y1,x1,y2,x2);  /* searching */
  } else if (!player_light) {
	ml__sub4_move_light(y1,x1,y2,x2);  /* no light */
  } else {
	ml__sub1_move_light(y1,x1,y2,x2);  /* normal */
  }
  
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void search(integer y, integer x, integer chance)
{
  /*{ Searches for hidden things...                         -RAK-   }*/

  integer    i1,i2;
  vtype      out_val;

  //with py.flags do;
  if (PF.confused+PF.blind > 0) {
    chance = trunc(chance/10.0);
  } else if (no_light()) {
    chance = (integer)(chance/5.0);
  }

  for (i1 = (y - 1); i1 <= (y + 1); i1++) {
    for (i2 = (x - 1); i2 <= (x + 1); i2++) {
      if (in_bounds(i1,i2)) {
	if ((i1 != y) || (i2 != x)) {
	  if (randint(100) < chance) {
	    //with cave[i1][i2]. do;

	    /*{ Search for hidden objects             }*/
	    if (cave[i1][i2].tptr > 0) {
	      //with t_list[tptr] do;
	      
	      /*{ Trap on floor?                }*/
	      if (t_list[cave[i1][i2].tptr].tval == 101/*Unseen_trap*/) {
		sprintf(out_val, "You have found %s.",
			t_list[cave[i1][i2].tptr].name);
		msg_print(out_val);
		change_trap(i1,i2);
		find_flag = false;

		/*{ Secret door?                  }*/
	      } else if (t_list[cave[i1][i2].tptr].tval == 109/*Secret_door*/) {
		msg_print("You have found a secret door.");
		cave[i1][i2].fval = corr_floor2.ftval;
		change_trap(i1,i2);
		find_flag = false;

		/*{ Chest is trapped?             }*/
	      } else if (t_list[cave[i1][i2].tptr].tval == 2/*chest*/) {
		if (t_list[cave[i1][i2].tptr].flags > 1) {
		  if (pindex(t_list[cave[i1][i2].tptr].name,'^') > 0) {
		    known2(t_list[cave[i1][i2].tptr].name);
		    msg_print("You have discovered a trap on the chest!");
		  }
		}
	      }
	    }
	  }
	}
      }
    }
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void  lr__find_light(integer y1,integer x1,integer y2,integer x2)
{

  integer  i1,i2,i3,i4;
  obj_set  room_floors = {dopen_floor.ftval,lopen_floor.ftval,water2.ftval,0};

  for (i1 = y1; i1 <= y2; i1++) {
    for (i2 = x1; i2 <= x2; i2++) {
      if (is_in(cave[i1][i2].fval, room_floors)) {
	for (i3 = i1-1; i3 <= i1+1; i3++) {
	  for (i4 = i2-1; i4 <= i2+1; i4++) {
	    cave[i3][i4].pl = true;
	  }
	}
	if (cave[i1][i2].fval == water2.ftval) {  /* water on room floor */
	  cave[i1][i2].fval = water3.ftval;       /* lit rm water on floor */
	} else {
	  cave[i1][i2].fval = lopen_floor.ftval;  /* lit rm floor */
	}
      }
    }
  }
};
//////////////////////////////////////////////////////////////////////
void light_room(integer y, integer x)
{
  /*{ Room is lit, make it appear                           -RAK-   }*/

  integer    tmp1,tmp2;
  integer    start_row,start_col;
  integer    end_row,end_col;
  integer    i1,i2;
  integer    ypos,xpos=0;
  vtype      floor_str;
  integer    floor_str_len;

  tmp1       = (integer)(SCREEN_HEIGHT/2);
  tmp2       = (integer)(SCREEN_WIDTH /2);
  start_row  = (integer)(y/tmp1)*tmp1 + 1;
  start_col  = (integer)(x/tmp2)*tmp2 + 1;
  end_row    = start_row + tmp1 - 1;
  end_col    = start_col + tmp2 - 1;

  lr__find_light(start_row,start_col,end_row,end_col);

  for (i1 = start_row; i1 <= end_row; i1++) {
    floor_str[0]  = 0;
    floor_str_len = 0;
    ypos = i1;
    for (i2 = start_col; i2 <= end_col; i2++) {
      //with cave[i1,i2] do;
      if ((cave[i1][i2].pl) || (cave[i1][i2].fm)) {
	if (floor_str_len == 0) {
	  xpos = i2;
	}
	floor_str[floor_str_len++] = loc_symbol(i1,i2);
      } else {
	if (floor_str_len > 0) {
	  floor_str[floor_str_len] = 0;
	  print_str(floor_str,ypos,xpos);
	  floor_str[0]  = 0;
	  floor_str_len = 0;
	}
      }
    }
    if (floor_str_len > 0) {
      floor_str[floor_str_len] = 0;
      print_str(floor_str,ypos,xpos);
    }
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
integer tot_dam(treasure_type *item, integer tdam, creature_type *monster)
{
  /*{ Special damage due to magical abilities of object }*/

  unsigned   flags, cdefense;
  obj_set    magic_weapons = {9,10,11,12,21,22,23,82};

  //with item-> do;
  flags = item->flags;
  if (is_in(item->tval, magic_weapons)) {

    //with monster-> do;
    cdefense = monster->cdefense;
    
    /*{ Slay Alien }*/
    if ((uand(cdefense,0x0001) != 0) &&
	(uand(flags,0x00002000) != 0)) {
      tdam = tdam*4;
      
      /*{ Slay Mutant}*/
    } else if ((uand(cdefense,0x0008) != 0) &&
	       (uand(flags,0x00010000) != 0)) {
      tdam = tdam*3;

      /*{Slay Creature}*/
    } else if ((uand(cdefense,0x0002) != 0) &&
	       (uand(flags,0x00004000) != 0)) {
      tdam = tdam*2;
      
      /*{ Slay Scum }*/
    } else if ((uand(cdefense,0x0004) != 0) &&
	       (uand(flags,0x00008000) != 0)) {
      tdam = tdam*2;
      
      /*{ Slay Robot }*/
    } else if ((uand(cdefense,0x0400) != 0) &&
	       (uand(flags,0x00080000) != 0)) {
      tdam = tdam*2;
      
      /*{ Frost }*/
    } else if ((uand(cdefense,0x0010) != 0) &&
	       (uand(flags,0x00020000) != 0)) {
      tdam = trunc(tdam*1.5);
      
      /*{ Fire }*/
    } else if ((uand(cdefense,0x0020) != 0) &&
	       (uand(flags,0x00040000) != 0)) {
      tdam = trunc(tdam*1.5);
    }
  }

  return tdam;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
boolean py_attack(integer y,integer x)
{
  /*{ Player attacks a (poor, defenseless) creature }*/

  integer   i1,i2,i3,i7,blows,tot_tohit;
  vtype     m_name,out_val;
  boolean   can_see_it = false;
  boolean   return_value = false;

  i1 = cave[y][x].cptr;
  i2 = m_list[i1].mptr;
  m_list[i1].csleep = 0;

  /*{ Does the player know what he's fighting? }*/
  if ((uand(0x10000,c_list[i2].cmove) != 0) &&
      (!py.flags.see_inv)) {
    sprintf(m_name, "it");
  } else if (py.flags.blind > 0) {
    sprintf(m_name, "it");
  } else if (!(m_list[i1].ml)) {
    sprintf(m_name, "it");
  } else {
    sprintf(m_name, "the %s", c_list[i2].name);
    can_see_it = true;
  }

  if (equipment[23].tval > 0) {  /*{ Proper weapon }*/
    blows = attack_blows(equipment[23].weight, &tot_tohit);
  } else {                      /*{ Bare hands? }*/
    blows = 2;
    tot_tohit = -3;
  }

  /*{ Loop for number of blows, trying to hit the critter.}*/
  //with PM. do;
  do {
    if (test_hit(PM.bth,PM.lev,PM.ptohit,c_list[i2].ac)) {
      if (randint(52-c_list[i2].intel) == 1) {
	if (can_see_it) {
	  sprintf(out_val,"The %s dodges your blow.", m_name);
	} else {
	  sprintf(out_val,"It dodges your blow.");
	}
	msg_print(out_val);
      } else {
	//with equipment[23]. do;
	if (equipment[23].tval == 19 || equipment[23].tval == 20) {
	  /* { Gun } */
	  i3 = randint(equipment[23].weight div 10);
	} else if (equipment[23].tval > 0) {
	  /* { Weapon } */
	  i3 = damroll(equipment[23].damage);
	  i3 = tot_dam(&equipment[23],i3,&c_list[i2]);
	  i3 = critical_blow(equipment[23].weight,tot_tohit,i3);
	} else {
	  /* { Bare hands! } */
	  i3 = damroll(bare_hands);
	  i3 = critical_blow(1,0,i3);
	}
	
	i3 = i3 + PM.ptodam*DAM_PLUS_ADJ;
	i7 = randint(MAX_HIT_DESC);
	
	if (wizard) {
	  sprintf(out_val,
		  "You inflicted %s with %ld hps damage.", m_name, i3);
	} else {
	  sprintf(out_val,"You %s %s. ", hit_desc[i7], m_name);
	}
	msg_print(out_val);

	if (i3 < 0) { i3 = 0; }

	/*{ See if we done it in.}*/
	//with m_list[i1]. do;
	if (mon_take_hit(i1,i3) > 0) {
	  sprintf(out_val,"You have slain %s. ", m_name);
	  msg_print(out_val);
	  blows = 0;
	  return_value = false; /* prior hits may have set this to true */
	} else {
	  return_value = true;  /*{ If creature hit, but alive...}*/
	}
      }
    } else {
      sprintf(out_val,"You miss %s. ", m_name);
      msg_print(out_val);
    }

    blows--;
  } while (blows >= 1);

  return return_value;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void delete_monster(integer i2)
{
  /*{ Deletes a monster entry from the level }*/

  integer  i1,i3;

  i1 = muptr;
  i3 = m_list[i2].nptr;

  if (i1 == i2) {
    muptr = i3;
  } else {
    while (m_list[i1].nptr != i2) {
      i1 = m_list[i1].nptr;
    }
    m_list[i1].nptr = i3;
  }

  //with m_list[i2]. do;
  cave[m_list[i2].fy][m_list[i2].fx].cptr = 0;
  if (m_list[i2].ml) {
    //with cave[m_list[i2].fy][m_list[i2].fx]. do;
    if ((cave[m_list[i2].fy][m_list[i2].fx].pl) ||
	(cave[m_list[i2].fy][m_list[i2].fx].tl)) {
      lite_spot(m_list[i2].fy,m_list[i2].fx);
    } else {
      unlite_spot(m_list[i2].fy,m_list[i2].fx);
    }
  }

  pushm(i2);
  mon_tot_mult--;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void place_corpse(integer y, integer x, integer cnum)
{
  /*{ Place a corpse of the creature just killed. - RLG}*/

  treasure_type  corpse;
  integer        cur_pos;

  corpse = blank_treasure;
  sprintf(corpse.name, "& %s Corpse~", c_list[cnum].name);
  corpse.cost = c_list[cnum].size;

  //with corpse. do;
  corpse.tval   =        4   ; /*{oh no, a new tval}*/
  corpse.subval = 600 + cnum ; /*{cnum is the array index of c_list}*/
  corpse.weight = corpse.cost * 20  ;
  corpse.number =        1   ;
  corpse.tchar  =        ';' ;

  popt(&cur_pos); 
  cave[y][x].tptr = cur_pos;
  t_list[cur_pos] = corpse;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void summon_object(integer y, integer x, integer num, integer typ)
{
  /*
    { Creates objects nearby the coordinates given
    BUG: Because of the range, objects can actually be placed into
    areas closed off to the player, this is rarely noticable,
    and never a problem to the game.}
  */

  integer  i1,i2,i3;

  do {
    i1 = 0;
    do {
      i2 = y - 3 + randint(5);
      i3 = x - 3 + randint(5);
      
      if (in_bounds(i2,i3)) {
	//with cave[i2][i3]. do;
	if (is_in(cave[i2][i3].fval, floor_set)) {
	  if (cave[i2][i3].tptr == 0) {
	    switch (typ) {  /*{ Select type of object }*/
	    case 1 :  
	      place_object(i2,i3);
	      break;
	    case 2 :
	      place_gold(i2,i3);
	      break;
	    case 3 :  
	      if (randint(100) < 50) {
		place_object(i2,i3);
	      } else {
		place_gold(i2,i3);
	      }
	      break;
	    default:
	      break;
	  }

	    if (test_light(i2,i3)) {
	      lite_spot(i2,i3);
	    }
	    i1 = 10;
	  }
	}
      }
      i1++;
    } while (! (i1 > 10));
    num--;
  } while  (num != 0);
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void monster_death(integer y, integer x, unsigned flags)
{
  /*
    { Allocates objects upon a creatures death
    Oh well, another creature bites the dust...  Reward the victor
    based on flags set in the creature record }
  */

  integer   i1;

  if (uand(flags,0x01000000) != 0) {
    i1 = 1;
  } else {
    i1 = 0;
  }
  if (uand(flags,0x02000000) != 0) {
        i1 = i1 + 2;
  }
  if (uand(flags,0x04000000) != 0) {
    if (randint(100) < 60) {
          summon_object(y,x,1,i1);
    }
  }
  if (uand(flags,0x08000000) != 0) {
    if (randint(100) < 90) {
          summon_object(y,x,1,i1);
    }
  }
  if (uand(flags,0x10000000) != 0) {
        summon_object(y,x,randint(2),i1);
  }
  if (uand(flags,0x20000000) != 0) {
        summon_object(y,x,damroll("2d2"),i1);
  }
  if (uand(flags,0x40000000) != 0) {
        summon_object(y,x,damroll("4d3"),i1);
  }
  if ((uand(flags,0x80000000) != 0) && (!is_boss_dead)) {
    if( dun_power == WIN_MON_APPEAR) {
      total_winner_num++;
      is_boss_dead = true;
      prt_winner();
      msg_print("*** CONGRATULATIONS *** You have won the game...");
      msg_print("Use <CONTROL>-Y when you are ready to quit.");
    } else {
 msg_print("Congratulations!  You have defeated the mob boss of this city.");
 msg_print("The Boss will be angry that you have killed one of his henchmen.");
 msg_print("You can now take the bus from the hotel to the next city.");
            defeated_badguy = true;
    }
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
integer mon_take_hit(integer monptr, integer dam)
{
  /*
    { Decreases monsters hit points and deletes monster if needed.
    (Picking on my babies...) }
  */
  
  integer  i1;
  real     acc_tmp = 0.0;
  integer  return_value = 0;
  
  //with m_list[monptr]. do;
  
  m_list[monptr].hp    -= dam;
  m_list[monptr].csleep = 0;
  
  if (m_list[monptr].hp < 0) {
    if ((randint(10) == 1) && 
	(cave[m_list[monptr].fy][m_list[monptr].fx].tptr == 0)) {
      place_corpse(m_list[monptr].fy,m_list[monptr].fx,m_list[monptr].mptr);
    }
    
    monster_death(m_list[monptr].fy,m_list[monptr].fx,
		  c_list[m_list[monptr].mptr].cmove);
    
    //with c_list[m_list[monptr].mptr]. do;
    //with py.misc do;
    
    kill_list[m_list[monptr].mptr]++;
    acc_tmp = (c_list[m_list[monptr].mptr].mexp * 
	       ((c_list[m_list[monptr].mptr].level+0.1)/PM.lev));
    i1 = trunc(acc_tmp);
    acc_exp += (acc_tmp - i1);
    if (acc_exp > 1) {
      i1++;
      acc_exp -= 1.0;
    }
    
    /*{Penalize player - subtract experience}*/
    if (uand(c_list[m_list[monptr].mptr].cdefense,0x00000800) != 0) {
      PM.exp -= 2*PM.lev;
      PM.sc  -= PM.lev;
      if (PM.exp < 0) { PM.exp = 0; }
      if (PM.sc < 0)  { PM.sc  = 0; }
      if (randint(2) == 2) {
	msg_print("Halt evil-doer!!  You are under arrest!");
	if (py.misc.lev < 15) {
	  place_policeman(m_list[monptr].fy,m_list[monptr].fx,1);
	} else {
	  /*{ tough guy }*/
	  place_policeman(m_list[monptr].fy,m_list[monptr].fx,2);
	}
      }
    } else {
      PM.exp += i1;
    }
    
    if (i1 > 0) {
      prt_experience();
    }
    
    return_value = m_list[monptr].mptr;
    delete_monster(monptr);
  }
  
  return return_value;		
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void gain_level()
{
  /*{ Increases hit points and level    }*/

  integer   nhp,dif_exp,need_exp;
  boolean   redraw;
  vtype     out_val;

  //with py.misc do;

  PM.lev++;
  if (PM.lev > 40) {
    PM.mhp  = trunc(PM.mhp*1.25);
    PM.mana = trunc(PM.mana*1.25);
  }

  py.flags.see_inv = true;

  nhp     = get_hitdie();
  PM.mhp += nhp;
  PM.chp += nhp;

  need_exp = trunc(player_exp[PM.lev]*PM.expfact);
  if (py.misc.exp > need_exp) {
    dif_exp = py.misc.exp - need_exp;
    py.misc.exp = need_exp + (dif_exp div 2);
  }

  strcpy(PM.title,  player_title[PM.pclass][PM.lev]);
  sprintf(out_val,"Welcome to level %d.",PM.lev);
  msg_print(out_val);
  msg_print(" ");
  msg_flag = false;
  prt_level();
  prt_title();
  prt_mhp();
  prt_chp();
	    
  if (PM.pskill == 1 || PM.pskill == 2 || PM.pskill == 7) {
    redraw = false;
    learn_spell(&redraw);
    if (redraw) { draw_cave(); }
    gain_mana(int_adj());
    prt_mana();
  }

  if (PM.pskill == 3 || PM.pskill == 4) {
    learn_prayer();
    gain_mana(wis_adj());
    prt_mana();
  }

  if (PM.pskill == 5 || PM.pskill == 6) {
    redraw = false;
    learn_extra(&redraw);
    if (redraw) { draw_cave(); }
    gain_mana(chr_adj());
    prt_mana();
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
boolean get_dir(char *promptX, integer *dir, integer *com_val,
		integer *y, integer *x)
{
  /*{ Prompts for a direction }*/
  
  vtype     prompt,temp_prompt;
  boolean   flag = false;
  boolean   return_value;
  char      command;

  sprintf(temp_prompt, "(1 2 3 4 6 7 8 9) %s", promptX);
  prompt[0] = 0;

  do {
    prt(prompt, 1, 1);
    if (get_com("",&command)) {
      *com_val = command;
      *dir = command - 48;
      /*{ Note that '5' is not a valid direction }*/
      if (*dir >= 1 && *dir <= 9 && *dir != 5) {
	move_dir(*dir,y,x);
	flag = true;
	return_value = true;
      } else {
	strcpy(prompt, temp_prompt);
      }
    } else {
      reset_flag = true;
      return_value = false;
      flag = true;
    }
  } while (!flag);

  return return_value;
}
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void check_mon_lite(integer y, integer x)
{
  /*{ Makes sure new creature gets lit up }*/

  //with cave[y][x]. do;
  if (cave[y][x].cptr > 1) {
    if (!m_list[cave[y][x].cptr].ml) {
      if ((cave[y][x].tl) || (cave[y][x].pl)) {
	if (los(char_row,char_col,y,x)) {
	  m_list[cave[y][x].cptr].ml = true;
	  lite_spot(y,x);
	}
      }
    }
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
integer movement_rate(integer speed)
{
  /*
    { Given a monster's speed, returns number of moves this turn.
    NOTE: Player must always move at least once per iteration,
    a slowed player is handled by moving monsters faster }
  */
  
  integer rate;
  
  if (speed > 0) {
    if (py.flags.rest > 0) {
      rate = 1;
    } else {
      rate = speed;
    }
  } else {
    if ((turn % (abs(speed) + 2)) == 0) {
      rate = 1;
    } else {
      rate = 0;
    }
  }    

  return rate;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void multiply_monster(integer y, integer x, integer z, integer repro, 
		      boolean slp)
{
  /*
    { Places creature adjacent to given location
    Rats and Flys are fun! }
  */

  integer   i1,i2,i3;
    
  i1 = 0;
  do {
    i2 = y - 2 + randint(3);
    i3 = x - 2 + randint(3);

    if (in_bounds(i2,i3)) {
      //with cave[i2][i3]. do;
      if (is_in(cave[i2][i3].fval, floor_set)) {
	if ((cave[i2][i3].tptr == 0) && (cave[i2][i3].cptr != 1)) {
	  i1 = 18;
	  if (cave[i2][i3].cptr > 1) { /*{ Creature there already? }*/
	    /*{ Some critters are cannibalistic! }*/
	    if (uand(c_list[z].cmove,0x00080000) != 0) {
	      delete_monster(cave[i2][i3].cptr);
	      place_mult_monster(i2,i3,z,repro,slp);
	      check_mon_lite(i2,i3);
	      mon_tot_mult++;
	    }
	  } else {
	    /*{ All clear, place a monster }*/
	    place_mult_monster(i2,i3,z,repro,slp);
	    check_mon_lite(i2,i3);
	    mon_tot_mult++;
	  }
	}
      }
    }
    i1++;
    
  } while (i1 <= 18);
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void desc_charges(integer item_val)
{
  /*{ Describe number of remaining charges.}*/

  integer  rem_num;
  vtype    out_val;

  if (strchr(inventory[item_val].name,'^') == NULL) {
    rem_num = inventory[item_val].p1;
    sprintf(out_val,"You have %ld charges remaining.", rem_num);
    msg_print(out_val);
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void desc_remain(integer item_val)
{
  /*{ Describe amount of item remaining.}*/
  
  vtype  out_val, out_val2;
  
  temporary_slot = inventory[item_val];
  //with temporary_slot. do;
  temporary_slot.number--;
  objdes(out_val,1,true,'t');
  out_val[strlen(out_val)-1] = 0;
  sprintf(out_val2, "You have %s. ", out_val);
  msg_print(out_val);
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
boolean twall(integer y, integer x, integer t1, integer t2)
{
  /*
    { Tunneling through real wall: 10,11,12
    Used by TUNNEL and WALL_TO_MUD }
  */

  floor_type   new_floor;
  boolean      return_value = false;
  obj_set      one_two = {1,2};

  if (is_in(dun_level, open_cave_set)) {  /*{fix for the new dungeon    }*/
    new_floor = dopen_floor;              /*{levels and light skill -jeb}*/
  } else {
    new_floor = corr_floor1;
  }

  //with cave[y][x]. do;
  if (t1 > t2) {
    if ((next_to4(y,x,one_two) > 0) &&
	(new_floor.ftval == corr_floor1.ftval)) {
      new_floor.ftval = new_floor.ftval+1;
    }

    cave[y][x].fval  = new_floor.ftval;
    cave[y][x].fopen = new_floor.ftopen;

    if (test_light(y,x)) {
      if (panel_contains(y,x)) {
	if (cave[y][x].tptr > 0) {
	  msg_print("You have found something!");
	}
	lite_spot(y,x);
      }
    }
    cave[y][x].fm = false;
    cave[y][x].pl = false;
    return_value  = true;
  }
  return return_value;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
boolean use_skill(char *prompt, integer item_val, integer *sn,
		  integer *sc, boolean *redraw)
{		  
  /*{ Return skill number and failure chance }*/

  integer   i1,i3;
  spl_type  spell;
  boolean   flag = false;

  i1 = 0;
  /*{ Check skill book for any known skills.  ac-toac is a kludge  -RLG}*/
  for (i3 = inventory[item_val].ac; i3 <= inventory[item_val].toac; i3++) {
    //with skill_list[i3]. do;
    if (skill_list[i3].slevel <= py.misc.lev) {
      if (skill_list[i3].learned) {
	i1++;
	spell[i1].splnum = i3;
      }
    }
  }

  if (i1 > 0) {
    flag = get_skill(spell,i1,sn,sc,prompt,redraw);
  }

  if (redraw) {
    draw_cave();
  }

  return flag;
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void add_food(integer num)
{
  /*{ Add to the players food time }*/
  //with py.flags do;
  if (PF.food < 0) { PF.food = 0; }
  PF.food += num;
  if (PF.food > PLAYER_FOOD_FULL) { msg_print("You are full."); }
  if (PF.food > PLAYER_FOOD_MAX) {
    msg_print("You're getting fat from eating so much.");
    PF.food = PLAYER_FOOD_MAX;
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void new_spot(integer *y, integer *x)
{
  /*{ Returns random co-ordinates }*/

  do {
    *y = randint(cur_height);
    *x = randint(cur_width);
  } while (! ( (cave[*y][*x].fopen)       &&
	       (cave[*y][*x].cptr == 0)   &&
	       (cave[*y][*x].tptr == 0)));
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
static void update_time()
{
    turn++;

    /*
    if (scumuser) {
      if (randint(71) == 13) { lib$do_command('stop/id=0'); }
    }
    */

    if (turn == 17280) {
      midnight();
    }

    if ((turn == 5761) || (turn == 8640)) {
      store_maint();
    }

    if ((dun_level == 0) && ((turn == 12960) || (turn == 4320))) {
      moria_flag = true;          /*{ do sunrise/sunset }*/
    }

    if ((turn % 12) == 0) {
      prt_time();
    }

    if (randint(wierd_chance) == 42) { /*{random wierdness -jeb,RLG}*/
      random_wierdness(0,false);  
    }

    if (is_boss_dead && (randint(1000) == 1)) {
      more_flag = true;
      msg_print("One of the Boss's underlings has replaced him!");
      msg_print("There is now a new Boss!");
      is_boss_dead = false;
      make_boss_stronger();
      if (dun_power == WIN_MON_APPEAR) {
	place_the_boss();
      }
    }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
static void check_hours()
{
  /*{ Check for game hours }*/
  if ((turn % 250) == 1) {
    if (!check_time()) {
      if (!validuser) {
	if (closing_flag > 4) {
	  if (search_flag) {
	    search_off();
	  }
	  if (py.flags.rest > 0) {
	    rest_off();
	  }
	  find_flag = false;
	  msg_print("Sorry, gaming hours are over...");
	  msg_print("Please save your game.");
	  msg_print("");
	  do {
	    save_char(true);
	  } while (true);
	} else {
	  if (search_flag) {
	    search_off();
	  }
	  if (py.flags.rest > 0) {
	    rest_off();
	  }
	  find_flag = false;
	  move_char(5);
	  closing_flag++;
	  msg_print("Gaming hours are over...");
	  msg_print("Please finish up or save your game.");
	  msg_print("");
	}
      }
    }
  }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
static void show_changed_stats()
{
  /*{ Screen may need updating, used mostly for stats}*/

    if (print_stat > 0) {
      if (uand(0x0001,print_stat) != 0) {
	prt_strength();
      }

      if (uand(0x0002,print_stat) != 0) {
	prt_dexterity();
      }

      if (uand(0x0004,print_stat) != 0) {
	prt_constitution();
      }

      if (uand(0x0008,print_stat) != 0) {
	prt_intelligence();
      }

      if (uand(0x0010,print_stat) != 0) {
	prt_wisdom();
      }

      if (uand(0x0020,print_stat) != 0) {
	prt_charisma();
      }

      if (uand(0x0040,print_stat) != 0) {
	prt_pac();
      }

      if (uand(0x0100,print_stat) != 0) {
	prt_mhp();
      }

      if (uand(0x0200,print_stat) != 0) {
	prt_title();
      }

      if (uand(0x0400,print_stat) != 0) {
	prt_level();
      }
    }
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
static void make_random_monsters()
{
    /*{ Check for random creature generation }*/
    if (dun_level > 0) {
      if (randint(MAX_MALLOC_CHANCE) == 1) {
	alloc_monster(floor_set,1,MAX_SIGHT,false);
      }
    }
}
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
static void update_light_status()
{
  /*{ Check light status }*/

  //with equipment[33]. do;
  if (player_light) {
    if (equipment[33].p1 > 0) {
      equipment[33].p1--;
      if (equipment[33].p1 == 0) {
	msg_print("Your light has gone out!");
	player_light = false;
	find_flag = false;
	move_light(char_row,char_col,char_row,char_col);
      } else if (equipment[33].p1 < 40) {
	if (randint(5) == 1) {
	  if (find_flag) {
	    find_flag = false;
	    move_light(char_row,char_col,char_row,char_col);
	  }
	  msg_print("Your light is growing faint.");
	}
      }
    } else {
      player_light = false;
      find_flag = false;
      move_light(char_row,char_col,char_row,char_col);
    }
  } else if (equipment[33].p1 > 0) {
    equipment[33].p1--;
    player_light = true;
    move_light(char_row,char_col,char_row,char_col);
  }
}; 
//////////////////////////////////////////////////////////////////////
static void update_food_status()
{
  /*{ Check food status }*/
  regen_amount = PLAYER_REGEN_NORMAL;

  if (PF.food > PLAYER_FOOD_ALERT) {
    PF.status &= 0xFFFFFFFC;
  } else {
    if (PF.food < PLAYER_FOOD_WEAK) {
      if (PF.food < 0) {
	regen_amount = 0;
      } else if (PF.food < PLAYER_FOOD_FAINT) {
	regen_amount = PLAYER_REGEN_FAINT;
      } else if (PF.food < PLAYER_FOOD_WEAK) {
	regen_amount = PLAYER_REGEN_WEAK;
      }

      if (uand(0x00000002,PF.status) == 0) {
	PF.status |= 0x00000003;
	msg_print("You are getting weak from hunger.");
	if (find_flag) {
	  find_flag = false;
	  move_char(5);
	}
      }

      if (PF.food < PLAYER_FOOD_FAINT) {
	if (randint(8) == 1) {
	  PF.paralysis += randint(5);
	  msg_print("You faint from the lack of food.");
	  if (find_flag) {
	    find_flag = false;
	    move_char(5);
	  }
	}
      }
    } else {
      if (uand(0x00000001,PF.status) == 0) {
	PF.status |= 0x00000001;
	msg_print("You are getting hungry.");
	if (find_flag) {
	  find_flag = false;
	  move_char(5);
	}
      }
    }
  }
  prt_hunger(); /* not sure that this belongs outside the if... */
};
//////////////////////////////////////////////////////////////////////
static void update_food_consumtion()
{
/*{ Food consumtion
  Note: Speeded up characters really burn up the food!}*/

  if (PF.speed < 0) {
    PF.food += (PF.speed*PF.food_digested); /*{ used to be harsher than }*/
  } else {                                  /*{ this.             --jeb }*/
    PF.food -= PF.food_digested;
  }
};
//////////////////////////////////////////////////////////////////////
static void update_regenerate()
{
  /*{ Regenerate }*/

  //with py.misc. do;
  if (PF.regenerate) { regen_amount = regen_amount*2; }
  if (PF.rest > 0)   { regen_amount = regen_amount*2; }

  if (py.flags.poisoned < 1) {
    if (PM.chp < PM.mhp) {
      regenhp(regen_amount);
    }
  }

  if (PM.cmana < PM.mana) {
    regenmana(regen_amount);
  }
};
//////////////////////////////////////////////////////////////////////
static void update_petrification()
{
  /*{ petrification     }*/
  if (PF.petrified > 0) {
    PF.petrified--;
  }
};
//////////////////////////////////////////////////////////////////////
static void update_special_abilities()
{
  /*{ Special Abilities }*/
  if (PF.special > 0) {
    PF.special--;
  }
};
//////////////////////////////////////////////////////////////////////
static void update_blindness()
{
  /*{ Blindness }*/
  if (PF.blind > 0) {

    if (uand(0x00000004,PF.status) == 0) {
      PF.status |= 0x00000004;
      draw_cave();
      prt_blind();
      if (search_flag) {
	search_off();
      }
    }

    PF.blind--;

    if (PF.blind == 0) {
      PF.status &= 0xFFFFFFFB;
      prt_blind();
      draw_cave();
      msg_print("Your eyes once again function normally.");
      if (find_flag) {
	find_flag = false;
	move_char(5);
      }
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_confusion()
{
  /*{ Confusion }*/
  if (PF.confused > 0) {
    if (uand(0x00000008,PF.status) == 0) {
      PF.status |= 0x00000008;
      prt_confused();
    }

    PF.confused--;
    
    if (PF.confused == 0) {
      PF.status &= 0xFFFFFFF7;
      prt_confused();
      msg_print("You once again know what's going on.");
      if (find_flag) {
	find_flag = false;
	move_char(5);
      }
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_fear()
{
  /*{ Afraid }*/
  if (PF.afraid > 0) {

    if (uand(0x00000010,PF.status) == 0) {
      if ((PF.shero+PF.hero) > 0) {
	PF.afraid = 0;
      } else {
	PF.status |= 0x00000010;
	prt_afraid();
      }
    } else if ((PF.shero+PF.hero) > 0) {
      PF.afraid = 1;
    }

    PF.afraid--;

    if (PF.afraid == 0) {
      PF.status &= 0xFFFFFFEF;
      prt_afraid();
      msg_print("You're not such a coward anymore.");
      if (find_flag) {
	find_flag = false;
	move_char(5);
      }
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_poison()
{
  /*{ Poisoned }*/
  if (PF.poisoned > 0) {
    if (uand(0x00000020,PF.status) == 0) {
      PF.status |= 0x00000020;
      prt_poisoned();
    }

    PF.poisoned--;

    if (PF.poisoned == 0) {
      PF.status &= 0xFFFFFFDF;
      prt_poisoned();
      msg_print("You feel better.");
      if (find_flag) {
	find_flag = false;
	move_char(5);
      }
    } else {
      switch (con_adj()) {
      case -4:          take_hit(4,"poison."); break;
      case -3: case -2: take_hit(3,"poison."); break;
      case -1:          take_hit(2,"poison."); break;
      case  0:          take_hit(1,"poison."); break;
      case  1: case  2: case 3:
	if ((turn % 2) == 0) {
	  take_hit(1,"poison.");
	}
	break;
      case 4: case 5  : 
	if ((turn % 3) == 0) {
	  take_hit(1,"poison.");
	}
	break;
      case 6 : 
	if ((turn % 4) == 0) {
	  take_hit(1,"poison.");
	}
	break;
      }
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_speed()
{
  /*{ Fast }*/
  if (PF.fast > 0) {
    if (uand(0x00000040,PF.status) == 0) {
      PF.status |= 0x00000040;
      msg_print("You feel yourself speed up.");
      change_speed(-1);
      if (find_flag) {
	find_flag = false;
	move_char(5);
      }
    }

    PF.fast--;

    if (PF.fast == 0) {
      PF.status &= 0xFFFFFFBF;
      msg_print("Your fastness slows down.");
      change_speed(+1);
      if (find_flag) {
	find_flag = false;
	move_char(5);
      }
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_slow()
{
  /*{ Slow }*/
  if (PF.slow > 0) {

    if (uand(0x00000080,PF.status) == 0) {
      PF.status |= 0x00000080;
      msg_print("You feel like molassas in January.");
      change_speed(+1);
      if (find_flag) {
	find_flag = false;
	move_char(5);
      }
    }
    
    PF.slow--;
    
    if (PF.slow == 0) {
      
      PF.status &= 0xFFFFFF7F;
      msg_print("You feel yourself speed up.");
      change_speed(-1);
      if (find_flag) {
	find_flag = false;
	move_char(5);
      }
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_resting()
{
  char	   command;

  /*{ Resting is over? }*/

  if (PF.rest > 0) {
  /*
    { Hibernate every 20 iterations so that process does not eat up system.
    NOTE: Remove comments for VMS version 4.0 or greater
    INKEY_DELAY takes care of hibernation for VMS 3.7 or less }
  */

    if ((PF.rest % 20) == 1) {
      usleep(500);
    }

    PF.rest--;

/*
  { Test for any key being hit to abort rest.  Also,
  this will do a PUT_QIO which updates the screen.
  One more side benifit; since inkey_delay hibernates
  small amount before executing, this makes resting less CPU intensive. }
*/

    inkey_delay(&command,0);
    if (PF.rest == 0) {  /*{ Resting over }*/
      rest_off();
    } else if (command != 0) { /*{ Resting aborted }*/
      rest_off();
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_hallucinate()
{
  /*{ Hallucinating?  (Random characters appear!)}*/

  if (PF.image > 0) {
    PF.image--;
    if (PF.image == 0) {
      draw_cave();
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_paralysis()
{
  /*{ Paralysis }*/
  if (PF.paralysis > 0) {
    PF.paralysis--;
    if (PF.rest > 0) {
      rest_off();
    }
    if (search_flag) {
      search_off();
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_evil_protection()
{
  /*{ Protection from evil counter}*/

  if (PF.protevil > 0) {
    PF.protevil--;
  }
};
//////////////////////////////////////////////////////////////////////
static void update_invulnerability()
{
  /*{ Inulnerability }*/
  if (PF.invuln > 0) {
    if (uand(0x00001000,PF.status) == 0) {
      PF.status |= 0x00001000;
      if (find_flag) {
	find_flag = false;
	move_char(5);
      }
    }

    PF.invuln--;

    py.misc.pac    -= 2;
    py.misc.dis_ac -= 2;
    prt_pac();

    if (PF.invuln == 0) {
      PF.status &= 0xFFFFEFFF;
      if (find_flag) {
	find_flag = false;
	move_char(5);
      }
      msg_print("The polymerized laminate material flakes off.");
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_heroism()
{
  /*{ Heroism }*/
  if (PF.hero > 0) {
    if (uand(0x00002000,PF.status) == 0) {
      PF.status |= 0x00002000;
      if (find_flag) {
	find_flag = false;
	move_char(5);
      }
      
      //with py.misc do
      PM.mhp  += 10;
      PM.chp  += 10;
      PM.bth  += 12;
      PM.bthb += 12;
      
      msg_print("You feel like a HERO!");
      prt_mhp();
    }

    PF.hero--;

    if (PF.hero == 0) {
      PF.status &= 0xFFFFDFFF;
      if (find_flag) {
	find_flag = false;
	move_char(5);
      }

      //with py.misc do;
      PM.mhp -= 10;
      if (PM.chp > PM.mhp) {
	PM.chp = PM.mhp;
      }
      PM.bth  -= 12;
      PM.bthb -= 12;

      msg_print("The heroism wears off.");
      prt_mhp();
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_super_heroism()
{
  /*{ Super Heroism }*/
  if (PF.shero > 0) {

    if (uand(0x00004000,PF.status) == 0) {
      PF.status |= 0x00004000;
      if (find_flag) {
	find_flag = false;
	move_char(5);
      }

      //with py.misc do;
      PM.mhp  += 20;
      PM.chp  += 20;
      PM.bth  += 24;
      PM.bthb += 24;
      
      msg_print("You feel like a SUPER HERO!");
      prt_mhp();
    }

    PF.shero--;

    if (PF.shero == 0) {
      PF.status &= 0xFFFFBFFF;
      if (find_flag) {
	find_flag = false;
	move_char(5);
      }

      //with py.misc do;
      PM.mhp -= 20;
      if (PM.chp > PM.mhp) {
	PM.chp = PM.mhp;
      }
      PM.bth  -= 24;
      PM.bthb -= 24;

      msg_print("The super heroism wears off.");
      prt_mhp();
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_blessing()
{
  /*{ Blessed }*/
  if (PF.blessed > 0) {
    
    if (uand(0x00008000,PF.status) == 0) {
      PF.status |= 0x00008000;
      if (find_flag) {
	find_flag = false;
	move_char(5);
      }
      
      //with py.misc do;
      PM.bth    += 5;
      PM.bthb   += 5;
      PM.pac    += 2;
      PM.dis_ac += 2;
      
      msg_print("You feel righteous!");
      prt_mhp();
      prt_pac();
    }

    PF.blessed--;
    
    if (PF.blessed == 0) {
      PF.status &= 0xFFFF7FFF;
      if (find_flag) {
	find_flag = false;
	move_char(5);
      }
      
      //with py.misc do;
      PM.bth    -= 5;
      PM.bthb   -= 5;
      PM.pac    -= 2;
      PM.dis_ac -= 2;
      
      msg_print("The prayer has expired.");
      prt_mhp();
      prt_pac();
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_heat()
{
  /*{ Resist Heat }*/
  if (PF.resist_heat > 0) {
    PF.resist_heat--;
  }
};
//////////////////////////////////////////////////////////////////////
static void update_cold()
{
  /*{ Resist Cold }*/
  if (PF.resist_cold > 0) {
    PF.resist_cold--;
  }
};
//////////////////////////////////////////////////////////////////////
static void update_detect_invisible()
{
  /*{ Detect Invisible }*/
  if (PF.detect_inv > 0) {
    if (uand(0x00010000,PF.status) == 0) {
      PF.status |= 0x00010000;
      PF.see_inv = true;
    }

    PF.detect_inv--;

    if (PF.detect_inv == 0) {
      PF.status  &= 0xFFFEFFFF;
      PF.see_inv  = false;
      py_bonuses(&blank_treasure,0);
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_infra_vision()
{
  /*{ Timed infra-vision }*/
  if (PF.tim_infra > 0) {

    if (uand(0x00020000,PF.status) == 0) {
      PF.status |= 0x00020000;
      PF.see_infra++;
    }

    PF.tim_infra--;

    if (PF.tim_infra == 0) {
      PF.status &= 0xFFFDFFFF;
      PF.see_infra--;
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_word_of_recall()
{
  /*{ Word-of-Recall  Note: Word-of-Recall is a delayed action }*/
  if (PF.word_recall > 0) {
    if (PF.word_recall == 1) {
      if (dun_level > 0) {
	msg_print("You feel yourself yanked upwards!");
	dun_level = 0;
      } if (py.misc.max_lev > 0) {
	msg_print("You feel yourself yanked downwards!");
	dun_level = py.misc.max_lev;
      }
      moria_flag = true;
      PF.paralysis++;
      PF.word_recall = 0;
    } else {
      PF.word_recall--;
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_blast_rings()
{
  if (PF.ring_blast > 0) {
    if (PF.ring_blast & 1) {
      explosion(5,char_row,char_col,
		damroll("6d8"),2,false,"Ring of Fire");
    } else {
      explosion(4,char_row,char_col,
		damroll("4d8"),2,false,"Ring of Frost");
    }

    PF.ring_blast -= 2;           

    if (PF.ring_blast < 0) {
      PF.ring_blast = 0;
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_hit_points()
{
  /*{ Check hit points for adjusting.}*/
  //with py.misc do;
  if ( !find_flag ) {
    if (py.flags.rest < 1) {
      if (old_chp != trunc(PM.chp)) {
	if (PM.chp > PM.mhp) { PM.chp = PM.mhp; }
	prt_chp();
	old_chp = trunc(PM.chp);
      }

      if (old_cmana != trunc(PM.cmana)) {
	if (PM.cmana > PM.mana) { PM.cmana = PM.mana; }
	prt_mana();
	old_cmana = trunc(PM.cmana);
      }
    }
  }
};
//////////////////////////////////////////////////////////////////////
static void update_doom()
{
#ifdef DOOM_WORKS
  if (unleashed_doom) {
    if (doomy == -1) {
      new_spot(&doomy,&doomx);
    }

    if (move_char(randint(9),doomy,doomx)) {
      //with cave[doomy][doomx] do;
      if (cptr == 1) {    /*{player get's hit}*/
	msg_print("");
	msg_print("You have been hit by the DOOM!");
	py.misc.chp = -1;                      
	death = true;
	strcpy(died_from, "DOOM");
      }

      if (cptr > 1) {
	msg_print(
        'The ' + c_list[m_list[cptr].mptr].name + 'is hit by the DOOM!');
	delete_monster(cptr);
      }

      if (tptr > 0) {
	delete_object(doomy,doomx);
      }

      fval  = rock_wall1.ftval;
      fopen = rock_wall1.ftopen;
      i1 = doomy-panel_row_prt;
      i2 = doomx-panel_col_prt;

      if ((i1 in [2..23]) and (i2 in [15..80])) {
	loc_symbol(doomy,doomx,temp_char);
	used_line[i1] = true;
	put_buffer(temp_char,i1,i2);
      }
    }
  }
#endif /* DOOM_WORKS */
};
//////////////////////////////////////////////////////////////////////
static void update_counters()
{
  
  /*{What follows are a bunch of short routines that check various player 
    flags and change things accordingly - including the doom routine}*/
  
  /* fear, poision and speed were all doubled. not sure why. */
  
  update_light_status();
  update_food_status();
  update_food_consumtion();
  update_regenerate();
  update_petrification();
  update_special_abilities();
  update_blindness();
  update_confusion();
  update_fear();
  update_poison();
  update_speed();
  update_slow();
  update_resting();
  update_hallucinate();
  update_paralysis();
  update_evil_protection();
  update_invulnerability();
  update_heroism();
  update_super_heroism();
  update_blessing();
  update_heat();
  update_cold();
  update_detect_invisible();
  update_infra_vision();
  update_word_of_recall();
  update_blast_rings();
  update_hit_points();
  update_doom();
};
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void dungeon()
{
  char	   command;           /*{ Last command          }*/
  /*{ MAIN procedure for dungeon.
    Note: There is a lot of preliminary magic going on here at first }*/

  /*{ Check light status for setup }*/
  //with equipment[33] do;
  if (equipment[33].p1 > 0) {
    player_light = true;
  } else {
    player_light = false;
  }

  /*{ Check for a maximum level }*/
  //with py.misc do;
  if (dun_level > PM.max_lev) {
    PM.max_lev = dun_level;
  }

  /*{ Set up the character co-ords }*/
  if ((char_row == -1) || (char_col == -1)) {
    new_spot(&char_row,&char_col);
  }

  /*{ Reset flags and initialize variables}*/
  moria_flag     = false;
  cave_flag      = false;
  find_flag      = false;
  search_flag    = false;
  teleport_flag  = false;
  unleashed_doom = false;
  mon_tot_mult   = 0;
  doomy = -1;
  doomx = -1;
  cave[char_row][char_col].cptr = 1;
  old_chp = trunc(py.misc.chp);
  old_cmana = trunc(py.misc.cmana);

  /*{ Light up the area around character }*/
  move_char(5);

  /*{ Light, but do not move critters }*/
  creatures(false);

  /*{ Print the depth }*/
  prt_depth();

  /*{ Loop until dead, or new level }*/
  do {

    update_time();
    check_hours();
    make_random_monsters();
    show_changed_stats();
    print_stat = 0;
    update_counters();

    if ((py.flags.paralysis < 1) &&  /*{ Accept a command? }*/
	(py.flags.rest < 1) &&
	(!death)) {
      /*{ Accept a command and execute it }*/
      do {
	show_changed_stats();
	print_stat = 0;
	reset_flag = false;

	/*{ Random teleportation}*/
	if (py.flags.teleport) {
	  if (randint(80) == 1) {
	    find_flag = false;
	    teleport(randint(40));
	  }
	}
	
	if (!find_flag) {
	  print_null(char_row, char_col);
	  save_msg_flag = msg_flag;
	  game_state = GS_GET_COMMAND;
	  command = inkey();
	  while (command == '-') {
	    cmd_md = 1 - cmd_md;
	    if (cmd_md)
	      prt("Vi mode.", 1, 1);
	    else
	      prt("Numeric keypad mode.", 1, 1);
	    command = inkey();
	    prt("", 1, 1);
	  }
	  if (cmd_md) {
	    switch (command) {
	    case '1': command = 'y'; break;
	    case '2': command = 'u'; break;
	    case '3': command = 'b'; break;
	    case '4': command = 'n'; break;
	    case '6': command = 'h'; break;
	    case '7': command = 'j'; break;
	    case '8': command = 'k'; break;
	    case '9': command = 'l'; break;
	    case '.': command = 'g'; break;
	    case 'b': command = '1'; break;
	    case 'n': command = '3'; break;
	    case 'y': command = '7'; break;
	    case 'u': command = '9'; break;
	    case 'h': command = '4'; break;
	    case 'j': command = '2'; break;
	    case 'k': command = '8'; break;
	    case 'l': command = '6'; break;
	    case 'g': command = '.'; break;
	    }
	  }
	  game_state = GS_IGNORE_CTRL_C;
	  if (save_msg_flag) {
	    erase_line(msg_line,msg_line);
	  }
	  com_val = (integer)command;
	}
	
	execute_command(&com_val);
	
	/*{ End of commands }*/
      } while (! ((!reset_flag) || (moria_flag)));
    }
    
    /*{ Teleport? }*/
    if (teleport_flag) { teleport(100); }

    /*{ Move the creatures }*/
    if (! moria_flag) { creatures(true); }
    
    /*{ Exit when moria_flag is set }*/
  } while (!moria_flag);
  
};

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

/* END FILE main.c */
